@@ -1475,26 +1475,28 @@ C***********************************************************************
 C
 C     CALCULATE DISTANCE ARRAYS
 C
-      WRITE(6,*)
+C *** it seems that this is not necessary...
+C      WRITE(6,*)
       PI=4.D0*ATAN(1.D0)
       IPX2=2*IPX
 C     IF THIS IS A RESTART RUN, READ IN RESTART INFO
-      IF(INDEX(KEYWRD,'ESPRST') .NE. 0) THEN
-         OPEN(UNIT=15,FILE='ESP.DUMP',STATUS='OLD',FORM='UNFORMATTED')
-         READ(15) JSTART,IESPS
-         IF(JSTART .EQ. ISC*2) THEN
-            CLOSE(15)
-            RETURN
-         ENDIF
-         DO 10 I=1,NESP
-            READ(15) ES(I)
-   10    CONTINUE
-         CLOSE(15)
-C
-         JSTART=JSTART+1
-      ELSE
+C *** skip all restart stuff, we don't need that...
+C      IF(INDEX(KEYWRD,'ESPRST') .NE. 0) THEN
+C         OPEN(UNIT=15,FILE='ESP.DUMP',STATUS='OLD',FORM='UNFORMATTED')
+C         READ(15) JSTART,IESPS
+C         IF(JSTART .EQ. ISC*2) THEN
+C            CLOSE(15)
+C            RETURN
+C         ENDIF
+C         DO 10 I=1,NESP
+C            READ(15) ES(I)
+C   10    CONTINUE
+C         CLOSE(15)
+CC
+C         JSTART=JSTART+1
+C      ELSE
          JSTART=1
-      ENDIF
+C      ENDIF
       NP=IS+1
       DO 200 IC=JSTART,ISC
          IPR=IC*ICD-ICD+1
@@ -1687,7 +1689,8 @@ C     SET NUMBER OF EQUALLY SPACED DUMPS
       IDN=10
 C
       IDC=0
-      WRITE(6,*)
+C *** it seems that this is not necessary...
+C      WRITE(6,*)
       IPX2=2*IPX
       PI=4.D0*ATAN(1.D0)
       NP=IS+1
@@ -1724,23 +1727,24 @@ C
 C
 C     READ IN RESTART INFORMATION IF THIS IS A RESTART
 C
-      IF(INDEX(KEYWRD,'ESPRST') .NE. 0) THEN
-         OPEN(UNIT=15,FILE='ESP.DUMP',STATUS='UNKNOWN',FORM='UNFORMATTED
-     1')
-         READ(15) JSTART,IESPS
-         IF(JSTART .NE. ISC*2) THEN
-            IESPS=0
-            CLOSE(15)
-            GOTO 50
-         ENDIF
-         DO 40 I=1,NESP
-            READ(15) ES(I)
-   40    CONTINUE
-         CLOSE(15)
-         IDC=FLOAT(IESPS)/FLOAT(NESP)*10
-      ELSE
+C *** skip all restart stuff, we don't need that...
+C      IF(INDEX(KEYWRD,'ESPRST') .NE. 0) THEN
+C         OPEN(UNIT=15,FILE='ESP.DUMP',STATUS='UNKNOWN',FORM='UNFORMATTED
+C     1')
+C         READ(15) JSTART,IESPS
+C         IF(JSTART .NE. ISC*2) THEN
+C            IESPS=0
+C            CLOSE(15)
+C            GOTO 50
+C         ENDIF
+C         DO 40 I=1,NESP
+C            READ(15) ES(I)
+C   40    CONTINUE
+C         CLOSE(15)
+C         IDC=FLOAT(IESPS)/FLOAT(NESP)*10
+C      ELSE
          IESPS=0
-      ENDIF
+C      ENDIF
    50 CONTINUE
 C
 C     LOOP OVER ESP PROBE POINTS
