@@ -136,7 +136,7 @@ struct {
 
 static integer c__1 = 1;
 static integer c__9 = 9;
-static doublereal c_b114 = 9999.;
+static doublereal c_b113 = 9999.;
 static integer c__45150 = 45150;
 
 /* Subroutine */ int iter_(h__, w, wj, wk, ee, fulscf, rand)
@@ -174,11 +174,9 @@ ACHIEVE SELF-CONSISTENCE\002,/)";
     integer i_indx(), f_open(), f_rew(), s_rsue(), do_uio(), e_rsue(), s_wsfe(
 	    ), do_fio(), e_wsfe();
     double sqrt();
-    integer s_wsle(), e_wsle();
-    /* Subroutine */ int s_stop();
-    integer do_lio();
+    integer s_wsle(), e_wsle(), do_lio();
     double d_sign();
-    integer f_clos(), s_rsfe(), e_rsfe();
+    integer f_clos();
 
     /* Local variables */
     extern /* Subroutine */ int diag_();
@@ -228,7 +226,6 @@ ACHIEVE SELF-CONSISTENCE\002,/)";
     extern /* Subroutine */ int vecprt_();
     extern doublereal second_();
     static doublereal shiftb, tenold, shftbo, tf1;
-    extern /* Subroutine */ int writmo_();
     extern doublereal helect_(), capcor_();
     static doublereal sellim;
     extern /* Subroutine */ int interp_(), matout_(), densit_();
@@ -263,20 +260,19 @@ BLE TO ACHIEVE SELF-CONSISTENCE, JOB CONTINUING')", 0 };
     static cilist io___95 = { 0, 6, 0, 0, 0 };
     static cilist io___96 = { 0, 6, 0, "(' ITERATION',I3,' PLS=',2E10.3,' EN\
 ERGY  ',   F14.7,' DELTAE',F13.7)", 0 };
-    static cilist io___98 = { 0, 6, 1, "()", 0 };
-    static cilist io___99 = { 0, 6, 0, "(2(A,F7.2))", 0 };
-    static cilist io___103 = { 0, 6, 0, "(2(A,F7.2))", 0 };
-    static cilist io___104 = { 0, 6, 0, "(//10X,A,                          \
+    static cilist io___98 = { 0, 6, 0, "(2(A,F7.2))", 0 };
+    static cilist io___102 = { 0, 6, 0, "(2(A,F7.2))", 0 };
+    static cilist io___103 = { 0, 6, 0, "(//10X,A,                          \
                    ' EIGENVECTORS AND EIGENVALUES ON ITERATION',I3)", 0 };
-    static cilist io___105 = { 0, 6, 0, fmt_330, 0 };
-    static cilist io___109 = { 0, 6, 0, "(//10X,A,' EIGENVECTORS AND EIGENVA\
+    static cilist io___104 = { 0, 6, 0, fmt_330, 0 };
+    static cilist io___108 = { 0, 6, 0, "(//10X,A,' EIGENVECTORS AND EIGENVA\
 LUES ON ',    'ITERATION',I3)", 0 };
-    static cilist io___110 = { 0, 6, 0, fmt_330, 0 };
-    static cilist io___111 = { 0, 6, 0, "(' DENSITY MATRIX ON ITERATION',I4)",
+    static cilist io___109 = { 0, 6, 0, fmt_330, 0 };
+    static cilist io___110 = { 0, 6, 0, "(' DENSITY MATRIX ON ITERATION',I4)",
 	     0 };
-    static cilist io___113 = { 0, 6, 0, "(27X,'AFTER MECI, ENERGY  ',F14.7)", 
+    static cilist io___112 = { 0, 6, 0, "(27X,'AFTER MECI, ENERGY  ',F14.7)", 
 	    0 };
-    static cilist io___114 = { 0, 6, 0, "(' NO. OF ITERATIONS =',I6)", 0 };
+    static cilist io___113 = { 0, 6, 0, "(' NO. OF ITERATIONS =',I6)", 0 };
 
 
 /* COMDECK SIZES */
@@ -981,10 +977,13 @@ L160:
 	do_fio(&c__1, (char *)&diff, (ftnlen)sizeof(doublereal));
 	do_fio(&c__1, (char *)&pl, (ftnlen)sizeof(doublereal));
 	e_wsfe();
-	mesage_1.iflepo = 9;
-	mesage_1.iiter = 2;
-	writmo_(&timec_1.time0, &escf);
-	s_stop("", (ftnlen)0);
+/* *** here we failed to calculate a valid energy, but we don't want to close the whole program either. */
+/* *** instead of calling STOP, continue like in the above case where GOTO 380 is called... */
+	goto L380;
+/*         IFLEPO=9 */
+/*         IITER=2 */
+/*         CALL WRITMO(TIME0,ESCF) */
+/*         STOP */
     }
     *ee = helect_(&molkst_1.norbs, densty_1.pa, &h__[1], fokmat_1.f);
     if (uhf) {
@@ -1142,7 +1141,7 @@ L320:
     }
     if (prtpl || debug && niter > itrmax - 20) {
 	if (abs(escf) > 99999.) {
-	    escf = d_sign(&c_b114, &escf);
+	    escf = d_sign(&c_b113, &escf);
 	}
 	if (abs(diff) > 9999.) {
 	    diff = 0.;
@@ -1162,30 +1161,13 @@ L320:
 	f_clos(&cl__1);
 /* ***** Modified by Jiro Toyoda at 1994-05-25 ***** */
 /*      OPEN(UNIT=6,FILE=GETNAM('FOR006'),ACCESS='APPEND') */
-	o__1.oerr = 0;
-	o__1.ounit = 6;
-	o__1.ofnmlen = 80;
-	getnam_(ch__1, (ftnlen)80, "FOR006", (ftnlen)6);
-	o__1.ofnm = ch__1;
-	o__1.orl = 0;
-	o__1.osta = 0;
-	o__1.oacc = 0;
-	o__1.ofm = 0;
-	o__1.oblnk = 0;
-	f_open(&o__1);
-L9990:
-	i__1 = s_rsfe(&io___98);
-	if (i__1 != 0) {
-	    goto L9999;
-	}
-	i__1 = e_rsfe();
-	if (i__1 != 0) {
-	    goto L9999;
-	}
-	goto L9990;
-L9999:
+/* *** exactly why do we want to open unit 6??? it's already open?!?!?! */
+/* *** we also remove this because we want use STDOUT for output... */
+/*      OPEN(UNIT=6,FILE=GETNAM('FOR006')) */
+/* 9990 read (6,'()',end=9999) */
+/*         goto 9990 */
+/* 9999 continue */
 /* ***************************** at 1994-05-25 ***** */
-	;
     }
     if (incitr) {
 	eold = escf;
@@ -1209,7 +1191,7 @@ L9999:
     makeb = TRUE_;
     if (timitr) {
 	t0 = second_();
-	s_wsfe(&io___99);
+	s_wsfe(&io___98);
 	do_fio(&c__1, " ADJUST DAMPER  INTEGRAL", (ftnlen)24);
 	d__1 = t0 - titer0;
 	do_fio(&c__1, (char *)&d__1, (ftnlen)sizeof(doublereal));
@@ -1248,7 +1230,7 @@ L9999:
 /* #      CALL TIMER('AFTER HQRII') */
 	if (timitr) {
 	    t1 = second_();
-	    s_wsfe(&io___103);
+	    s_wsfe(&io___102);
 	    do_fio(&c__1, "  HQRII:", (ftnlen)8);
 	    d__1 = t1 - t0;
 	    do_fio(&c__1, (char *)&d__1, (ftnlen)sizeof(doublereal));
@@ -1264,7 +1246,7 @@ L9999:
 	if (uhf) {
 	    j = 2;
 	}
-	s_wsfe(&io___104);
+	s_wsfe(&io___103);
 	do_fio(&c__1, abprt + (j - 1) * 5, (ftnlen)5);
 	do_fio(&c__1, (char *)&niter, (ftnlen)sizeof(integer));
 	e_wsfe();
@@ -1272,7 +1254,7 @@ L9999:
 		 &molkst_1.norbs);
     } else {
 	if (prteig) {
-	    s_wsfe(&io___105);
+	    s_wsfe(&io___104);
 	    do_fio(&c__1, abprt + (j - 1) * 5, (ftnlen)5);
 	    do_fio(&c__1, (char *)&niter, (ftnlen)sizeof(integer));
 	    i__1 = molkst_1.norbs;
@@ -1363,7 +1345,7 @@ L340:
 		    vector_1.eigb, vector_1.cbeta);
 	}
 	if (prtvec) {
-	    s_wsfe(&io___109);
+	    s_wsfe(&io___108);
 	    do_fio(&c__1, abprt + 10, (ftnlen)5);
 	    do_fio(&c__1, (char *)&niter, (ftnlen)sizeof(integer));
 	    e_wsfe();
@@ -1371,7 +1353,7 @@ L340:
 		    molkst_1.norbs, &molkst_1.norbs);
 	} else {
 	    if (prteig) {
-		s_wsfe(&io___110);
+		s_wsfe(&io___109);
 		do_fio(&c__1, abprt + 10, (ftnlen)5);
 		do_fio(&c__1, (char *)&niter, (ftnlen)sizeof(integer));
 		i__1 = molkst_1.norbs;
@@ -1415,7 +1397,7 @@ L350:
 	}
     }
     if (prtden) {
-	s_wsfe(&io___111);
+	s_wsfe(&io___110);
 	do_fio(&c__1, (char *)&niter, (ftnlen)sizeof(integer));
 	e_wsfe();
 	vecprt_(densty_1.p, &molkst_1.norbs);
@@ -1490,7 +1472,7 @@ L380:
 	    *ee += sum;
 	    if (prtpl) {
 		escf = (*ee + enuclr_1.enuclr) * 23.061 + atheat_1.atheat;
-		s_wsfe(&io___113);
+		s_wsfe(&io___112);
 		do_fio(&c__1, (char *)&escf, (ftnlen)sizeof(doublereal));
 		e_wsfe();
 	    }
@@ -1498,7 +1480,7 @@ L380:
     }
     ++numscf_1.nscf;
     if (debug) {
-	s_wsfe(&io___114);
+	s_wsfe(&io___113);
 	do_fio(&c__1, (char *)&niter, (ftnlen)sizeof(integer));
 	e_wsfe();
     }
