@@ -15,7 +15,7 @@ using namespace std;
 
 /*################################################################################################*/
 
-mm1_eng_exp9::mm1_eng_exp9(mm1_mdl & p1, i32s p2, i32s p3) : mm1_eng(p1, p2, p3)
+mm1_eng_exp9::mm1_eng_exp9(mm1_mdl & p1) : mm1_eng(p1)
 {
 	exp9_tables::GetInstance()->UpdateTypes(GetModel());
 	
@@ -148,8 +148,11 @@ mm1_eng_exp9::mm1_eng_exp9(mm1_mdl & p1, i32s p2, i32s p3) : mm1_eng(p1, p2, p3)
 					i32s bt1[2] = { bt2_vector[tmp1].index1[0], bt2_vector[tmp1].index1[1] };
 					i32s bt2[2] = { bt2_vector[tmp1].dir1[0], bt2_vector[tmp1].dir1[1] };
 					
-					if (bt1[0] == ind1a[n1] && bt2[0] == dir1a[n1] && bt1[1] == (* it2).bndr->index && bt2[1] == another) tmp2 = true;
-					if (bt1[1] == ind1a[n1] && bt2[1] == dir1a[n1] && bt1[0] == (* it2).bndr->index && bt2[0] == another) tmp2 = false;
+					if (bt1[0] == ind1a[n1] && bt2[0] == dir1a[n1] &&
+						bt1[1] == (* it2).bndr->index && bt2[1] == another) tmp2 = true;
+					if (bt1[1] == ind1a[n1] && bt2[1] == dir1a[n1] &&
+						bt1[0] == (* it2).bndr->index && bt2[0] == another) tmp2 = false;
+					
 					if (tmp2 < 0) tmp1++; else break;
 				}
 				
@@ -175,8 +178,11 @@ mm1_eng_exp9::mm1_eng_exp9(mm1_mdl & p1, i32s p2, i32s p3) : mm1_eng(p1, p2, p3)
 					i32s bt1[2] = { bt2_vector[tmp1].index1[0], bt2_vector[tmp1].index1[1] };
 					i32s bt2[2] = { bt2_vector[tmp1].dir1[0], bt2_vector[tmp1].dir1[1] };
 					
-					if (bt1[0] == ind1b[n1] && bt2[0] == dir1b[n1] && bt1[1] == (* it2).bndr->index && bt2[1] != another) tmp2 = false;
-					if (bt1[1] == ind1b[n1] && bt2[1] == dir1b[n1] && bt1[0] == (* it2).bndr->index && bt2[0] != another) tmp2 = true;
+					if (bt1[0] == ind1b[n1] && bt2[0] == dir1b[n1] &&
+						bt1[1] == (* it2).bndr->index && bt2[1] != another) tmp2 = false;
+					if (bt1[1] == ind1b[n1] && bt2[1] == dir1b[n1] &&
+						bt1[0] == (* it2).bndr->index && bt2[0] != another) tmp2 = true;
+					
 					if (tmp2 < 0) tmp1++; else break;
 				}
 				
@@ -230,8 +236,9 @@ mm1_eng_exp9::mm1_eng_exp9(mm1_mdl & p1, i32s p2, i32s p3) : mm1_eng(p1, p2, p3)
 					bt[2] = (* itb3).bt.GetValue();
 					
 	bt3_err += !exp9_tables::GetInstance()->Init(this, & newbt3, bt);
-	if (newbt3.k == 0.0) continue;		// these have bad optimal geometries -> SKIP!!!
-	
+//	if (newbt3.k == 0.0) continue;		// these have bad optimal geometries -> SKIP!!! constraints???
+
+					newbt3.constraint = false;
 					bt3_vector.push_back(newbt3);
 				}
 			}
@@ -324,16 +331,6 @@ mm1_eng_exp9::mm1_eng_exp9(mm1_mdl & p1, i32s p2, i32s p3) : mm1_eng(p1, p2, p3)
 		char eee; cin >> eee;
 		if (eee == 'q' || eee == 'Q') exit(EXIT_FAILURE);
 	}
-	
-	// load-balancing?!?!?!
-	// load-balancing?!?!?!
-	// load-balancing?!?!?!
-
-	bt1_range[0] = 0; bt1_range[1] = bt1_vector.size();
-	bt2_range[0] = 0; bt2_range[1] = bt2_vector.size();
-	bt3_range[0] = 0; bt3_range[1] = bt3_vector.size();
-	
-	nbt1_range[0] = 0; nbt1_range[1] = nbt1_vector.size();
 }
 
 mm1_eng_exp9::~mm1_eng_exp9(void)
@@ -342,6 +339,91 @@ mm1_eng_exp9::~mm1_eng_exp9(void)
 	delete[] bt2data;
 }
 
+i32s mm1_eng_exp9::FindTorsion(mm1_atom * a1, mm1_atom * a2, mm1_atom * a3, mm1_atom * a4)
+{
+	i32s tmp[4] = { a1->index, a2->index, a3->index, a4->index };
+	
+	for (i32s n1 = 0;n1 < (i32s) bt3_vector.size();n1++)
+	{
+		bool test;	// since torsion is the same in both directions, we can ignore direction...
+		
+		test = true;
+		if (bt3_vector[n1].atmi[0] != tmp[0]) test = false;
+		if (bt3_vector[n1].atmi[1] != tmp[1]) test = false;
+		if (bt3_vector[n1].atmi[2] != tmp[2]) test = false;
+		if (bt3_vector[n1].atmi[3] != tmp[3]) test = false;
+		if (test) return n1;
+		
+		test = true;
+		if (bt3_vector[n1].atmi[0] != tmp[3]) test = false;
+		if (bt3_vector[n1].atmi[1] != tmp[2]) test = false;
+		if (bt3_vector[n1].atmi[2] != tmp[1]) test = false;
+		if (bt3_vector[n1].atmi[3] != tmp[0]) test = false;
+		if (test) return n1;
+	}
+	
+	return NOT_DEFINED;
+}
+
+bool mm1_eng_exp9::SetTorsionConstraint(i32s ind_bt3, f64 opt, f64 fc, bool lock_local_structure)
+{
+	if (ind_bt3 < 0) return false;
+	if (ind_bt3 >= (i32s) bt3_vector.size()) return false;
+	
+	// check that opt is in valid range [-pi,+pi]!!!
+	
+	while (opt > +M_PI) opt -= 2.0 * M_PI;
+	while (opt < -M_PI) opt += 2.0 * M_PI;
+	
+	// measure the current torsion and set constraints for the other torsions, if requested...
+	
+	if (lock_local_structure)
+	{
+		v3d<f64> v1a(crd[bt3_vector[ind_bt3].atmi[1]], crd[bt3_vector[ind_bt3].atmi[0]]);
+		v3d<f64> v1b(crd[bt3_vector[ind_bt3].atmi[1]], crd[bt3_vector[ind_bt3].atmi[2]]);
+		v3d<f64> v1c(crd[bt3_vector[ind_bt3].atmi[2]], crd[bt3_vector[ind_bt3].atmi[3]]);
+		f64 delta = opt - v1a.tor(v1b, v1c);
+		
+		while (delta > +M_PI) delta -= 2.0 * M_PI;
+		while (delta < -M_PI) delta += 2.0 * M_PI;
+		
+		i32s tmp1 = bt3_vector[ind_bt3].atmi[1];
+		i32s tmp2 = bt3_vector[ind_bt3].atmi[2];
+		
+		for (i32s n1 = 0;n1 < (i32s) bt3_vector.size();n1++)
+		{
+			bool test1 = true;
+			if (bt3_vector[n1].atmi[1] != tmp1) test1 = false;
+			if (bt3_vector[n1].atmi[2] != tmp2) test1 = false;
+			
+			bool test2 = true;
+			if (bt3_vector[n1].atmi[1] != tmp2) test2 = false;
+			if (bt3_vector[n1].atmi[2] != tmp1) test2 = false;
+			
+			if (test1 || test2)
+			{
+				v3d<f64> v2a(crd[bt3_vector[n1].atmi[1]], crd[bt3_vector[n1].atmi[0]]);
+				v3d<f64> v2b(crd[bt3_vector[n1].atmi[1]], crd[bt3_vector[n1].atmi[2]]);
+				v3d<f64> v2c(crd[bt3_vector[n1].atmi[2]], crd[bt3_vector[n1].atmi[3]]);
+				f64 local = v2a.tor(v2b, v2c) + delta;
+				
+				while (local > +M_PI) local -= 2.0 * M_PI;
+				while (local < -M_PI) local += 2.0 * M_PI;
+				
+				bt3_vector[n1].constraint = true;
+				bt3_vector[n1].k = local; bt3_vector[n1].s = fc;
+			}
+		}
+	}
+	
+	// then set the requested constraint... if lock_local_structure was true, then
+	// this operation is in fact redundant, but perhaps a bit more accurate than above.
+	
+	bt3_vector[ind_bt3].constraint = true;
+	bt3_vector[ind_bt3].k = opt; bt3_vector[ind_bt3].s = fc;
+	return true;
+}
+
 void mm1_eng_exp9::ComputeBT1(i32s p1)
 {
 	energy_bt1 = 0.0;
@@ -357,7 +439,7 @@ void mm1_eng_exp9::ComputeBT1(i32s p1)
 	
 	// the end point gradient of a vector is always the same as components of the unit vector!!!
 	
-	for (i32s n1 = bt1_range[0];n1 < bt1_range[1];n1++)
+	for (i32s n1 = 0;n1 < (i32s) bt1_vector.size();n1++)
 	{
 		i32s * atmi = bt1_vector[n1].atmi;
 		
@@ -417,7 +499,7 @@ void mm1_eng_exp9::ComputeBT2(i32s p1)
 	// dang[1] -> grad[0-2]: for atom 1 when direction = 0, for atom 1 when direction = 1
 	// dang[2] -> grad[0-2]: for atom 0 when direction = 0, for atom 2 when direction = 1
 	
-	for (i32s n1 = bt2_range[0];n1 < bt2_range[1];n1++)
+	for (i32s n1 = 0;n1 < (i32s) bt2_vector.size();n1++)
 	{
 		i32s * atmi = bt2_vector[n1].atmi;
 		
@@ -485,7 +567,7 @@ void mm1_eng_exp9::ComputeBT3(i32s p1)
 {
 	energy_bt3 = 0.0;
 	
-	for (i32s n1 = bt3_range[0];n1 < bt3_range[1];n1++)
+	for (i32s n1 = 0;n1 < (i32s) bt3_vector.size();n1++)
 	{
 		i32s * atmi = bt3_vector[n1].atmi;
 		
@@ -518,9 +600,12 @@ void mm1_eng_exp9::ComputeBT3(i32s p1)
 		// now we still have to determine the sign of the result...
 		
 		f64 t1g[3];
-		t1g[0] = bt1data[index1[2]].dlen[dir1[2]][1] * bt1data[index1[3]].dlen[dir1[3]][2] - bt1data[index1[2]].dlen[dir1[2]][2] * bt1data[index1[3]].dlen[dir1[3]][1];
-		t1g[1] = bt1data[index1[2]].dlen[dir1[2]][2] * bt1data[index1[3]].dlen[dir1[3]][0] - bt1data[index1[2]].dlen[dir1[2]][0] * bt1data[index1[3]].dlen[dir1[3]][2];
-		t1g[2] = bt1data[index1[2]].dlen[dir1[2]][0] * bt1data[index1[3]].dlen[dir1[3]][1] - bt1data[index1[2]].dlen[dir1[2]][1] * bt1data[index1[3]].dlen[dir1[3]][0];
+		t1g[0] = bt1data[index1[2]].dlen[dir1[2]][1] * bt1data[index1[3]].dlen[dir1[3]][2] -
+			bt1data[index1[2]].dlen[dir1[2]][2] * bt1data[index1[3]].dlen[dir1[3]][1];
+		t1g[1] = bt1data[index1[2]].dlen[dir1[2]][2] * bt1data[index1[3]].dlen[dir1[3]][0] -
+			bt1data[index1[2]].dlen[dir1[2]][0] * bt1data[index1[3]].dlen[dir1[3]][2];
+		t1g[2] = bt1data[index1[2]].dlen[dir1[2]][0] * bt1data[index1[3]].dlen[dir1[3]][1] -
+			bt1data[index1[2]].dlen[dir1[2]][1] * bt1data[index1[3]].dlen[dir1[3]][0];
 		
 		f64 t1h = t1c[0][0] * t1g[0] + t1c[0][1] * t1g[1] + t1c[0][2] * t1g[2];
 		if (t1h < 0.0) t1f[0] = -t1f[0];
@@ -528,9 +613,50 @@ void mm1_eng_exp9::ComputeBT3(i32s p1)
 		t1f[1] = t1f[0] + t1f[0];
 		t1f[2] = t1f[1] + t1f[0];
 		
-		// f = a(1+cos(x))+b(1-cos(2x))+c(1+cos(3x))
-		// df/dx = -a*sin(x)+2b*sin(2x)-3c*sin(3x)
-		
+		f64 t9a; f64 t9b;
+		if (bt3_vector[n1].constraint)
+		{
+			// Dx = x-b			| for the following formulas...
+			
+			// f = a(2PI-Dx)^4		| if Dx > +PI
+			// df/fx = -4a(2PI-Dx)^3
+			
+			// f = a(2PI+Dx)^4		| if Dx < -PI
+			// df/fx = +4a(2PI+Dx)^3
+			
+			// f = a(Dx)^4			| otherwise
+			// df/fx = 4a(Dx)^3
+			
+			f64 t8a = t1f[0] - bt3_vector[n1].k;
+			if (t8a > +M_PI)
+			{
+				t8a = 2.0 * M_PI - t8a;
+				f64 t8b = t8a * t8a;
+				
+				t9a = bt3_vector[n1].s * t8b * t8b;
+				t9b = -4.0 * bt3_vector[n1].s * t8b * t8a;
+			}
+			else if (t8a < -M_PI)
+			{
+				t8a = 2.0 * M_PI + t8a;
+				f64 t8b = t8a * t8a;
+				
+				t9a = bt3_vector[n1].s * t8b * t8b;
+				t9b = +4.0 * bt3_vector[n1].s * t8b * t8a;
+			}
+			else
+			{
+				f64 t8b = t8a * t8a;
+				
+				t9a = bt3_vector[n1].s * t8b * t8b;
+				t9b = 4.0 * bt3_vector[n1].s * t8b * t8a;
+			}
+		}
+		else
+		{
+			// f = a(1+cos(x))+b(1-cos(2x))+c(1+cos(3x))
+			// df/dx = -a*sin(x)+2b*sin(2x)-3c*sin(3x)
+			
 		// here we make a quick conversion into a fourier series...
 		// here we make a quick conversion into a fourier series...
 		// here we make a quick conversion into a fourier series...
@@ -539,20 +665,21 @@ void mm1_eng_exp9::ComputeBT3(i32s p1)
 		i32u zzz2 = abs(zzz1); if (zzz2 < 1) zzz2 = 1; zzz2 -= 1; if (zzz2 == 1) zzz1 = -zzz1;
 		fc[zzz2] = bt3_vector[n1].k; if (zzz1 < 0) fc[zzz2] = -fc[zzz2];
 		
-		f64 t8a = fc[0] * (1.0 + cos(t1f[0]));
-		f64 t8b = fc[1] * (1.0 - cos(t1f[1]));
-		f64 t8c = fc[2] * (1.0 + cos(t1f[2]));
-		f64 t8d = t8a + t8b + t8c;
+			f64 t8a = fc[0] * (1.0 + cos(t1f[0]));
+			f64 t8b = fc[1] * (1.0 - cos(t1f[1]));
+			f64 t8c = fc[2] * (1.0 + cos(t1f[2]));
+			t9a = t8a + t8b + t8c;
+			
+			f64 t8r = fc[0] * sin(t1f[0]);
+			f64 t8s = fc[1] * sin(t1f[1]) * 2.0;
+			f64 t8t = fc[2] * sin(t1f[2]) * 3.0;
+			t9b = t8s - (t8r + t8t);
+		}
 		
-		energy_bt3 += t8d;
+		energy_bt3 += t9a;
 		
 		if (p1 > 0)
 		{
-			f64 t9a = fc[0] * sin(t1f[0]);
-			f64 t9b = fc[1] * sin(t1f[1]) * 2.0;
-			f64 t9c = fc[2] * sin(t1f[2]) * 3.0;
-			f64 t9d = t9b - (t9a + t9c);
-			
 			f64 t2a = bt1data[index1[0]].len * t1b[0];
 			f64 t2b = bt1data[index1[0]].len * t1a[0] / bt1data[index1[1]].len;
 			
@@ -572,24 +699,29 @@ void mm1_eng_exp9::ComputeBT3(i32s p1)
 				f64 t5b = bt1data[index1[2]].dlen[dir1[2]][cp[n2][1]] * bt1data[index1[3]].dlen[dir1[3]][cp[n2][2]];
 				t5c[n2] = (t5a - t5b) / t3a;
 				
-				d1[atmi[0]][n2] += t4c[n2] * t9d;
-				d1[atmi[3]][n2] += t5c[n2] * t9d;
+				d1[atmi[0]][n2] += t4c[n2] * t9b;
+				d1[atmi[3]][n2] += t5c[n2] * t9b;
 				
 				t6a[n2] = (t2b - 1.0) * t4c[n2] - t3b * t5c[n2];
 				t7a[n2] = (t3b - 1.0) * t5c[n2] - t2b * t4c[n2];
 				
-				d1[atmi[1]][n2] += t6a[n2] * t9d;
-				d1[atmi[2]][n2] += t7a[n2] * t9d;
+				d1[atmi[1]][n2] += t6a[n2] * t9b;
+				d1[atmi[2]][n2] += t7a[n2] * t9b;
 			}
 		}
 	}
 }
 
+void mm1_eng_exp9::ComputeBT4(i32s)
+{
+	energy_bt4 = 0.0;
+}
+
 void mm1_eng_exp9::ComputeNBT1(i32s p1)
 {
 	energy_nbt1 = 0.0;
 	
-	for (i32s n1 = nbt1_range[0];n1 < nbt1_range[1];n1++)
+	for (i32s n1 = 0;n1 < (i32s) nbt1_vector.size();n1++)
 	{
 		i32s * atmi = nbt1_vector[n1].atmi;
 		
