@@ -26,7 +26,7 @@ mm1_sequencebuilder mm1_mdl::amino_builder(MM1_AMINO_BUILDER_FILE);
 mm1_sequencebuilder mm1_mdl::nucleic_builder(MM1_NUCLEIC_BUILDER_FILE);
 
 mm1_mdl::mm1_mdl(ostream * p1, class_factory & p2) :
-	model_extended(p1, p2), model_simple(p1, p2), trajectory_interface()
+	model_extended(p1, p2), model_simple(p1, p2), all_atoms_interface(), trajectory_interface()
 {
 	nmol = NOT_DEFINED;
 	ref_civ = NULL;
@@ -52,7 +52,7 @@ const char * mm1_mdl::GetProjectFileNameExtension(void)
 
 const char * mm1_mdl::engtab1[] =
 {
-	"mm1_eng_exp9 : the normal engine",
+	"mm1_eng_exp1 : the normal engine",
 	"mm1_eng_exp1_mim : the periodic engine (minimum image model)",
 	NULL
 };
@@ -70,11 +70,12 @@ mm1_eng * mm1_mdl::CreateDefaultEngine(void)
 	{
 		case ENG_MM1_PERIODIC:
 		periodic = true; GatherGroups();
-		return new mm1_eng_exp1_mim((* this), 1, 0);
+		return new mm1_eng_exp1_mim(* this);
 		
 		default:	// ENG_MM1_NORMAL
 		periodic = false;
-		return new mm1_eng_exp9((* this), 1, 0);
+		return new mm1_eng_exp1(* this);
+		//return new mm1_eng_exp9(* this);	// experimental!!! for debugging only!!!
 	}
 }
 
@@ -143,6 +144,9 @@ cout << "Oops!!! This function is not yet ready." << endl;
 //exit(EXIT_FAILURE);
 }
 
+/*##############################################*/
+/*##############################################*/
+
 void mm1_mdl::AddAtom(mm1_atom & p1)
 {
 	InvalidateGroups();
@@ -197,6 +201,9 @@ void mm1_mdl::RemoveBond(iter_mm1bl it1)
 	bond_list.erase(it1);
 }
 
+/*##############################################*/
+/*##############################################*/
+
 bool mm1_mdl::ReadStream_OLD(istream & p1)
 {
 	static const char message[] = "Hmmm. It seems that we are trying to read a VERY OLD version of this file.\nIt's not possible to verify the type and version of the file, so this\noperation may fail. Are you sure you wish to try (Y/N) ??? ";
@@ -235,10 +242,13 @@ cout << "mm1_mdl::ReadStream_OLD() : starting file input." << endl;
 	return true;
 }
 
-bool mm1_mdl::ReadStream(istream & p1)
+bool mm1_mdl::ReadStream(istream & p1, bool selected)
 {
 	int version;
 	
+	if (bond_list.empty() && atom_list.empty())
+	  selected = false;
+
 	// first we have to identify the file version, and then call the matching function,
 	// if necessary. the very first file format did not use the "!Section" idea, so
 	// we must skip the test if no such traces are seen...
@@ -253,17 +263,6 @@ bool mm1_mdl::ReadStream(istream & p1)
 		p1 >> buffer;		// read the file type.
 		p1 >> version;		// read the version number;
 		
-		// just return false if new have incorrect file type...
-		
-		//		if (strcmp(buffer, GetProjectFileNameExtension()))
-		//		{
-		//#ifdef VERBOSE
-		//cout << "mm1_mdl::ReadStream() : incorrect file type : " << buffer << " != " << GetProjectFileNameExtension() << endl;
-		//#endif	// VERBOSE
-		//
-		//			return false;
-		//		}
-		
 		p1.getline(buffer, sizeof(buffer));
 	}
 	
@@ -278,9 +277,8 @@ bool mm1_mdl::ReadStream(istream & p1)
 		if (!result) return false;
 	}
 	
-	// the "!Header" section is now processed, and the actual file input begins!!!
-	// the "!Header" section is now processed, and the actual file input begins!!!
-	// the "!Header" section is now processed, and the actual file input begins!!!
+	// the "!Header" section is now processed,
+	// and the actual file input begins!!!
 	
 #ifdef VERBOSE
 cout << "mm1_mdl::ReadStream() : starting file input." << endl;
@@ -336,8 +334,9 @@ if (n1 != ar_vector.size()) cout << "WARNING : bad atom id numbers!!!" << endl;
 
 				element tmp_element(n2);
 				mm1_atom newatom(tmp_element, NULL, cs_vector.size());
-				
+				if (selected) newatom.selected = true;
 				AddAtom(newatom);
+
 				ar_vector.push_back(& atom_list.back());
 				
 				p1.getline(buffer, sizeof(buffer));
@@ -593,14 +592,43 @@ void mm1_mdl::WriteStream(ostream & p1)
 	p1 << "!End" << endl;
 }
 
+bool mm1_mdl::ImportFile(const char *filename, int index)
+{
+  ifstream ifile;
+  ostrstream intermed;
+  file_trans translator;
+  
+  if (index == 0) // Automatic detection
+    {
+      if (!translator.CanImport(filename))
+	{
+	  err->ErrorMessage("Cannot import that file type.");
+	  return false;
+	}
+      
+      ifile.open(filename, ios::in);
+      translator.Import(filename, ifile, intermed);
+      ifile.close();
+    }
+  else // By type picked by the user
+    {
+      ifile.open(filename, ios::in);
+      translator.Import(filename, index - 1, ifile, intermed);
+      ifile.close();
+    }
+
+  istrstream interInput(intermed.str());
+  return ReadStream(interInput);
+}
+
 void mm1_mdl::AddHydrogens(void)
 {
-	// this will use OELib to add hydrogens!!!
-        // works fine, but makes Kekule structures out of conjugated bonds!
+  // this will use OpenBabel to add hydrogens!!!
+  // works fine, but makes Kekule structures out of conjugated bonds!
 	
   	file_trans ft;
-  	OEMol *oem = ft.CopyAll(this);
-  	oem->AddHydrogens(false, false);
+  	OBMol *obm = ft.CopyAll(this);
+	obm->AddHydrogens(false, false);
   	ft.Synchronize(); return;
 	
 	// The old version can be found in the CVS tree
@@ -626,23 +654,47 @@ void mm1_mdl::RemoveHydrogens(void)
 	}
 }
 
-void mm1_mdl::Clear(void)
+iter_mm1al mm1_mdl::FindAtomByIndex(i32s index)
 {
-	iter_mm1bl it1 = bond_list.begin();
-	while (it1 != bond_list.end())
+	iter_mm1al end_it = GetAtomsEnd();
+	if (index < 0) return end_it;
+	
+	i32s counter = 0;
+	iter_mm1al iter = GetAtomsBegin();
+	while (counter != index)
 	{
-	  iter_mm1bl it2 = it1++;
-	  RemoveBond(it2);
+		counter++; iter++;
+		if (iter == end_it) return end_it;
 	}
 	
-	iter_mm1al it2 = atom_list.begin();
-	while (it2 != atom_list.end())
-	{
-	  iter_mm1al it3 = it2++;
-	  RemoveAtom(it3);
-	}
+	return iter;
 }
 
+void mm1_mdl::Clear(void)
+{
+	if (!bond_list.empty())
+	  {
+	    iter_mm1bl it1 = bond_list.begin();
+	    while (it1 != bond_list.end())
+	      {
+		iter_mm1bl it2 = it1++;
+		RemoveBond(it2);
+	      }
+	  }
+
+	if (!atom_list.empty())
+	  {
+	    iter_mm1al it2 = atom_list.begin();
+	    while (it2 != atom_list.end())
+	      {
+		iter_mm1al it3 = it2++;
+		RemoveAtom(it3);
+	      }
+	  }
+}
+
+/*##############################################*/
+/*##############################################*/
 
 void mm1_mdl::GatherGroups(void)
 {
@@ -769,6 +821,9 @@ void mm1_mdl::GatherAtoms(mm1_atom * ref, i32s id)
 	while (it1 != ref->cr_list.end()) GatherAtoms((* it1++).atmr, id);
 }
 
+/*##############################################*/
+/*##############################################*/
+
 // what comes to PDB chain id's here, we follow the "pdb-select principle" by converting
 // the empty id ' ' with '_' to improve readability...
 
@@ -1596,6 +1651,227 @@ void mm1_mdl::readpdb_ReadData_sub2(vector<mm1_readpdb_data_atom> & adata, i32s
 	AddBond(newbond);
 }
 
+/*##############################################*/
+/*##############################################*/
+
+iter_mm1al mm1_mdl::aai_FindA(i32s atmi) //const
+{
+	if (atmi < 0) return GetAtomsEnd();				// out of valid range...
+	if (atmi >= (i32s) aai_GetAtomCount()) return GetAtomsEnd();	// out of valid range...
+	
+	i32s counter = 0;
+	iter_mm1al it1 = GetAtomsBegin();
+	
+	while (counter < atmi)
+	{
+		counter++;
+		it1++;
+	}
+	
+	return it1;
+}
+
+iter_mm1bl mm1_mdl::aai_FindB(i32s bndi) //const
+{
+	if (bndi < 0) return GetBondsEnd();				// out of valid range...
+	if (bndi >= (i32s) aai_GetBondCount()) return GetBondsEnd();	// out of valid range...
+	
+	i32s counter = 0;
+	iter_mm1bl it1 = GetBondsBegin();
+	
+	while (counter < bndi)
+	{
+		counter++;
+		it1++;
+	}
+	
+	return it1;
+}
+
+i32u mm1_mdl::aai_GetAtomCount(void) //const
+{
+	return atom_list.size();
+}
+
+i32u mm1_mdl::aai_GetBondCount(void) //const
+{
+	return bond_list.size();
+}
+
+void * mm1_mdl::aai_FindAtomByInd(i32s atmi) //const
+{
+	iter_mm1al it1 = aai_FindA(atmi);
+	if (it1 == GetAtomsEnd()) return NULL;
+	else return (void *) & (* it1);
+}
+
+i32s mm1_mdl::aai_FindAtomByPtr(void * atmr) //const
+{
+	i32s counter = 0;
+	iter_mm1al it1 = GetAtomsBegin();
+	
+	while (it1 != GetAtomsEnd())
+	{
+		void * tmpr = & (* it1);
+		if (atmr == tmpr) return counter;
+		
+		counter++;
+		it1++;
+	}
+	
+	return -1;	// not found...
+}
+
+i32s mm1_mdl::aai_FindBond(i32s atmi1, i32s atmi2) //const
+{
+	void * atmr1 = aai_FindAtomByInd(atmi1);
+	if (atmr1 == NULL) return -1;
+	
+	void * atmr2 = aai_FindAtomByInd(atmi2);
+	if (atmr2 == NULL) return -1;
+	
+	i32s counter = 0;
+	iter_mm1bl it1 = GetBondsBegin();
+	
+	while (it1 != GetBondsEnd())
+	{
+		void * tmpr1 = (* it1).atmr[0];
+		void * tmpr2 = (* it1).atmr[1];
+		
+		if (atmr1 == tmpr1 && atmr2 == tmpr2) return counter;
+		if (atmr1 == tmpr2 && atmr2 == tmpr1) return counter;
+		
+		counter++;
+		it1++;
+	}
+	
+	return -1;	// not found...
+}
+
+void * mm1_mdl::aai_AddAtom(element el, fGL * xyz)
+{
+	mm1_atom newatom(el, xyz, cs_vector.size());
+	AddAtom(newatom);
+	
+	return (void *) & atom_list.back();
+}
+
+bool mm1_mdl::aai_RemoveAtom(i32s atmi)
+{
+	iter_mm1al it1 = aai_FindA(atmi);
+	if (it1 == GetAtomsEnd()) return false;
+	
+	RemoveAtom(it1);
+	
+	return true;
+}
+
+bool mm1_mdl::aai_AddBond(i32s atmi1, i32s atmi2, bondtype bt)
+{
+	void * atmr1 = aai_FindAtomByInd(atmi1);
+	if (atmr1 == NULL) return false;
+	
+	void * atmr2 = aai_FindAtomByInd(atmi2);
+	if (atmr2 == NULL) return false;
+	
+	mm1_bond newbond((mm1_atom *) atmr1, (mm1_atom *) atmr2, bt);
+	AddBond(newbond);
+	
+	return true;
+}
+
+bool mm1_mdl::aai_RemoveBond(i32s bndi)
+{
+	iter_mm1bl it1 = aai_FindB(bndi);
+	if (it1 == GetBondsEnd()) return false;
+	
+	RemoveBond(it1);
+	
+	return true;
+}
+
+bool mm1_mdl::aai_AtomGetElement(i32s atmi, element & el) //const
+{
+	iter_mm1al it1 = aai_FindA(atmi);
+	if (it1 == GetAtomsEnd()) return false;
+	
+	el = (* it1).el;
+	
+	return true;
+}
+
+bool mm1_mdl::aai_AtomSetElement(i32s atmi, element el)
+{
+	iter_mm1al it1 = aai_FindA(atmi);
+	if (it1 == GetAtomsEnd()) return false;
+	
+	(* it1).el = el;
+	
+	return true;
+}
+
+bool mm1_mdl::aai_AtomGetXYZ(i32s atmi, fGL * xyz) //const
+{
+	iter_mm1al it1 = aai_FindA(atmi);
+	if (it1 == GetAtomsEnd()) return false;
+	
+	xyz[0] = (* it1).crd_vector[0].data[0];
+	xyz[1] = (* it1).crd_vector[0].data[1];
+	xyz[2] = (* it1).crd_vector[0].data[2];
+	
+	return true;
+}
+
+bool mm1_mdl::aai_AtomSetXYZ(i32s atmi, const fGL * xyz)
+{
+	iter_mm1al it1 = aai_FindA(atmi);
+	if (it1 == GetAtomsEnd()) return false;
+	
+//	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
+	i32s n1 = 0;	// how to handle this properly?!?!?!
+	{
+		(* it1).crd_vector[n1].data[0] = xyz[0];
+		(* it1).crd_vector[n1].data[1] = xyz[1];
+		(* it1).crd_vector[n1].data[2] = xyz[2];
+	}
+	
+	return true;
+}
+
+bool mm1_mdl::aai_BondGetBondType(i32s bndi, bondtype & bt) //const
+{
+	iter_mm1bl it1 = aai_FindB(bndi);
+	if (it1 == GetBondsEnd()) return false;
+	
+	bt = (* it1).bt;
+	
+	return true;
+}
+
+bool mm1_mdl::aai_BondSetBondType(i32s bndi, bondtype bt)
+{
+	iter_mm1bl it1 = aai_FindB(bndi);
+	if (it1 == GetBondsEnd()) return false;
+	
+	(* it1).bt = bt;
+	
+	return true;
+}
+
+bool mm1_mdl::aai_BondGetAtomIndices(i32s bndi, i32s * iii) //const
+{
+	iter_mm1bl it1 = aai_FindB(bndi);
+	if (it1 == GetBondsEnd()) return false;
+	
+	iii[0] = aai_FindAtomByPtr((void *) (* it1).atmr[0]);
+	iii[1] = aai_FindAtomByPtr((void *) (* it1).atmr[1]);
+	
+	return true;
+}
+
+/*##############################################*/
+/*##############################################*/
+
 void mm1_mdl::DoEnergy(void)
 {
 	mm1_eng * eng = CreateDefaultEngine();
@@ -1614,27 +1890,88 @@ void mm1_mdl::DoEnergy(void)
 
 void mm1_mdl::DoGeomOpt(mm1_geomopt_param & param)
 {
-  char buffer[1024];
 	GeomOptGetParam(param);
 	if (!param.confirm) return;
 	
 	mm1_eng * eng = CreateDefaultEngine();
-
 	CopyCRD(this, eng, 0);
 	
 	mm1_geomopt * opt = new mm1_geomopt(eng, 100, 0.025);		// optimal settings?!?!?
 	
-	for (i32s n1 = 0;n1 < param.nsteps;n1++)
+	char buffer[1024];
+	f64  last_energy = 0.0;		// this is for output and delta_e test...
+	
+	cout << "Cycle     Energy   Gradient     Step      Delta E" << endl;
+	cout << "--------------------------------------------------" << endl;
+	
+	i32s n1 = 0;	// n1 counts the number of steps...
+	while (true)
 	{
 		opt->TakeCGStep(conjugate_gradient::Newton2An);
-		sprintf(buffer, "%d %10.4f  %4.6e", n1, opt->optval, opt->optstp);
+		
+// problem: the gradient information is in fact not precise in this stage. the current gradient
+// is the one that was last calculated in the search, and it is not necessarily the best one.
+// to update the gradient, we need to Compute(1) here...
+	eng->Compute(1);	// this is not vital, but will update the gradient vector length...
+		
+		if (n1 != 0)
+		  {
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e %10.4e", n1,
+			    opt->optval, eng->GetGradientVectorLength(),
+			    opt->optstp, last_energy - opt->optval);
+		  }
+		else
+		  {
+		    sprintf(buffer, "%4d %12.5f %10.4e %10.4e **********", n1,
+			    opt->optval, eng->GetGradientVectorLength(),
+			    opt->optstp);
+		  }
 		cout << buffer << endl;
 		
-		if (!(n1 % 10) || (n1 + 1) == param.nsteps)
+		bool terminate = false;
+		
+		if (param.enable_nsteps)	// the nsteps test...
+		{
+			if (n1 >= param.treshold_nsteps)
+			{
+				terminate = true;
+				cout << "the nsteps termination test was passed." << endl;
+			}
+		}
+		
+		if (param.enable_grad)		// the grad test...
+		{
+			if (eng->GetGradientVectorLength() < param.treshold_grad)
+			{
+				terminate = true;
+				cout << "the grad termination test was passed." << endl;
+			}
+		}
+		
+		if (param.enable_delta_e)	// the delta_e test...
+		{
+			bool flag = false; const f64 treshold_step = 1.0e-12;		// can we keep this as a constant???
+			if (n1 != 0 && (last_energy - opt->optval) != 0.0 && fabs(last_energy - opt->optval) < param.treshold_delta_e) flag = true;
+			if ((opt->optstp != 0.0) && (opt->optstp < treshold_step)) flag = true;
+			
+			if (flag)
+			{
+				terminate = true;
+				cout << "the delta_e termination test was passed." << endl;
+			}
+		}
+		
+		last_energy = opt->optval;
+		
+		if (!(n1 % 10) || terminate)
 		{
 			CopyCRD(eng, this, 0); CenterCRDSet(0);
 			UpdateAllGraphicsViews(true);
 		}
+		
+		if (terminate) break;		// exit the loop here!!!
+		
+		n1++;	// update the number of steps...
 	}
 	
 	delete opt;
@@ -1790,8 +2127,17 @@ void mm1_mdl::DoFormula()
   ostrstream str(buffer, sizeof(buffer));
 
   int count[ELEMENT_SYMBOLS];
-  element lookup;
-  
+  // These are the atomic numbers of the elements in alphabetical order.
+  const int alphabetical[ELEMENT_SYMBOLS] = {
+   89, 47, 13, 95, 18, 33, 85, 79, 5, 56, 4, 107, 83, 97, 35, 6, 20, 48,
+   58, 98, 17, 96, 27, 24, 55, 29, 105, 66, 68, 99, 63, 9, 26, 100, 87, 31,
+   64, 32, 1, 2, 72, 80, 67, 108, 53, 49, 77, 19, 36, 57, 3, 103, 71, 101,
+   12, 25, 42, 109, 7, 11, 41, 60, 10, 28, 102, 93, 8, 76, 15, 91, 82, 46, 
+   61, 84, 59, 78, 94, 88, 37, 75, 104, 45, 86, 44, 16, 51, 21, 34, 106, 14, 
+   62, 50, 38, 73, 65, 43, 52, 90, 22, 81, 69, 92, 110, 23, 74, 54, 39, 70, 
+   30, 40 };
+  int index;
+
   for (i = 0; i < ELEMENT_SYMBOLS; i++)
     {
       count[i] = 0;
@@ -1805,24 +2151,26 @@ void mm1_mdl::DoFormula()
       molweight += (* it3).el.GetAtomicMass();
     }
   
-  ++lookup;
   for (i = 0; i < ELEMENT_SYMBOLS; i++)
     {
-      ++lookup;
-      if (count[i] > 1)
+      index = alphabetical[i] - 1;
+      if (count[index] > 1)
 	{
-	  str << lookup.GetSymbol() << count[i] << " ";
+	  str << (element(index + 1)).GetSymbol() << count[index] << " ";
 	}
-      else if (count[i] == 1)
+      else if (count[index] == 1)
 	{
-	  str << lookup.GetSymbol();
+	  str << (element(index + 1)).GetSymbol();
 	}
     }
 
-  str << "  MW: " << molweight << ends;
+  str << "MW: " << molweight << ends;
   err->Message(buffer);
 }
 
+/*##############################################*/
+/*##############################################*/
+
 void mm1_mdl::OpenTrajectory(const char * fn)
 {
 	if (!trajfile)
@@ -1879,6 +2227,7 @@ void mm1_mdl::ReadFrame(void)
 	}
 }
 
+/*################################################################################################*/
 /*################################################################################################*/
 
 // f = sum[Q/r] = sum[Q * r^-1]
@@ -1963,6 +2312,93 @@ fGL mm1_GetVDWSValue(fGL * crd, mm1_mdl * ref, fGL * grad)
 	return vdwsv;
 }
 
+/*################################################################################################*/
+/*################################################################################################*/
+
+// this is taken form mm2_superimpose...
+// this is taken form mm2_superimpose...
+// this is taken form mm2_superimpose...
+
+mm1_superimpose::mm1_superimpose(mm1_mdl * p1, i32s p2, i32s p3) : superimpose(p2, p3)
+{
+	mdl = p1;
+}
+
+mm1_superimpose::~mm1_superimpose(void)
+{
+}
+
+f64 mm1_superimpose::GetValue(void)
+{
+	value = 0.0; counter = 0;
+	
+	for (iter_mm1al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
+	{
+		f64 d1[3];
+		d1[0] = (* it1).crd_vector[index[0]].data[0];
+		d1[1] = (* it1).crd_vector[index[0]].data[1];
+		d1[2] = (* it1).crd_vector[index[0]].data[2];
+		
+		f64 d2[3];
+		d2[0] = (* it1).crd_vector[index[1]].data[0];
+		d2[1] = (* it1).crd_vector[index[1]].data[1];
+		d2[2] = (* it1).crd_vector[index[1]].data[2];
+		
+		Compare(d1, d2, false);
+	}
+	
+	return value;
+}
+
+f64 mm1_superimpose::GetGradient(void)
+{
+	value = 0.0; counter = 0;
+	dloc[0] = dloc[1] = dloc[2] = 0.0;
+	drot[0] = drot[1] = drot[2] = 0.0;
+	
+	for (iter_mm1al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
+	{
+		f64 d1[3];
+		d1[0] = (* it1).crd_vector[index[0]].data[0];
+		d1[1] = (* it1).crd_vector[index[0]].data[1];
+		d1[2] = (* it1).crd_vector[index[0]].data[2];
+		
+		f64 d2[3];
+		d2[0] = (* it1).crd_vector[index[1]].data[0];
+		d2[1] = (* it1).crd_vector[index[1]].data[1];
+		d2[2] = (* it1).crd_vector[index[1]].data[2];
+		
+		Compare(d1, d2, true);
+	}
+	
+	return value;
+}
+
+void mm1_superimpose::Transform(void)
+{
+	value = 0.0; counter = 0;
+	
+	for (iter_mm1al it1 = mdl->GetAtomsBegin();it1 != mdl->GetAtomsEnd();it1++)
+	{
+		f64 d1[3];
+		d1[0] = (* it1).crd_vector[index[0]].data[0];
+		d1[1] = (* it1).crd_vector[index[0]].data[1];
+		d1[2] = (* it1).crd_vector[index[0]].data[2];
+		
+		f64 d2[3];
+		d2[0] = (* it1).crd_vector[index[1]].data[0];
+		d2[1] = (* it1).crd_vector[index[1]].data[1];
+		d2[2] = (* it1).crd_vector[index[1]].data[2];
+		
+		f64 d3[3];
+		Compare(d1, d2, false, d3);
+		
+		(* it1).crd_vector[index[1]].data[0] = d3[0];
+		(* it1).crd_vector[index[1]].data[1] = d3[1];
+		(* it1).crd_vector[index[1]].data[2] = d3[2];
+	}
+}
+
 /*################################################################################################*/
 
 // eof
