@@ -333,7 +333,8 @@ i32s mm1_typerule::ReadSubRule(istream * istr, ostream * ostr)
 				}
 				
 				char tmp2[3] = { 0, 0, 0 }; tmp2[0] = (char) istr->get();
-				if (istr->peek() >= 'a' && istr->peek() <= 'z') tmp2[1] = (char) istr->get(); newsr.el = element(tmp2);
+				if (istr->peek() >= 'a' && istr->peek() <= 'z') tmp2[1] = (char) istr->get();
+				newsr.el = element(tmp2);
 				
 				bool problem2 = (newsr.el.GetAtomicNumber() == NOT_DEFINED && tmp2[0] != '*');
 				if (problem2 && ostr != NULL)
@@ -367,8 +368,10 @@ bool mm1_typerule::CheckRules(mm1_mdl * mdl, mm1_atom * atom, i32s flag, i32s ru
 		for (n1 = false, it1 = atom->cr_list.begin();!n1 && it1 != atom->cr_list.end();it1++)
 		{
 			if ((* it1).bndr->flags[flag]) continue;
-			if (sr_vector[rule].el.GetAtomicNumber() != NOT_DEFINED && sr_vector[rule].el.GetAtomicNumber() != (* it1).atmr->el.GetAtomicNumber()) continue;
-			if (sr_vector[rule].bt.GetValue() != NOT_DEFINED && sr_vector[rule].bt.GetValue() != (* it1).bndr->bt.GetValue()) continue;
+			if (sr_vector[rule].el.GetAtomicNumber() != NOT_DEFINED &&
+				sr_vector[rule].el.GetAtomicNumber() != (* it1).atmr->el.GetAtomicNumber()) continue;
+			if (sr_vector[rule].bt.GetValue() != NOT_DEFINED &&
+				sr_vector[rule].bt.GetValue() != (* it1).bndr->bt.GetValue()) continue;
 			
 			(* it1).bndr->flags[flag] = true; n1 = true;
 			if (n1 && (sr_vector[rule].sub != NOT_DEFINED)) n1 = CheckRules(mdl, (* it1).atmr, flag, sr_vector[rule].sub);
