@@ -78,6 +78,79 @@ const char * mm2_mdl::GetProjectFileNameExtension(void)
 	return ext;
 }
 
+/*##############################################*/
+/*##############################################*/
+
+void mm2_mdl::PushCRDSets(i32u p1)
+{
+	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());
+	
+	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
+	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+	{
+		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+		{
+			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+			{
+				for (i32u n4 = 0;n4 < p1;n4++)
+				{
+					chn_vector[n1].res_vector[n2].crd_vector[n3].push_back(newcrd);
+				}
+			}
+			
+			for (i32u n3 = 0;n3 < p1;n3++)
+			{
+				chn_vector[n1].res_vector[n2].peptide_vector.push_back(0.0);
+			}
+		}
+	}
+}
+
+void mm2_mdl::PopCRDSets(i32u)
+{
+cout << "Oops!!! This function is not yet ready." << endl;
+//exit(EXIT_FAILURE);
+}
+
+void mm2_mdl::CopyCRDSet(i32u p1, i32u p2)
+{
+	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
+	{
+		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
+		{
+			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
+			{
+	chn_vector[n1].res_vector[n2].crd_vector[n3][p2][0] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][0];
+	chn_vector[n1].res_vector[n2].crd_vector[n3][p2][1] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][1];
+	chn_vector[n1].res_vector[n2].crd_vector[n3][p2][2] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][2];
+			}
+			
+	chn_vector[n1].res_vector[n2].peptide_vector[p2] = chn_vector[n1].res_vector[n2].peptide_vector[p1];
+		}
+	}
+}
+
+void mm2_mdl::SwapCRDSets(i32u, i32u)
+{
+cout << "Oops!!! This function is not yet ready." << endl;
+//exit(EXIT_FAILURE);
+}
+
+void mm2_mdl::CenterCRDSet(i32u)
+{
+cout << "Oops!!! This function is not yet ready." << endl;
+//exit(EXIT_FAILURE);
+}
+
+void mm2_mdl::ReserveCRDSets(i32u)
+{
+cout << "Oops!!! This function is not yet ready." << endl;
+//exit(EXIT_FAILURE);
+}
+
+/*##############################################*/
+/*##############################################*/
+
 i32s mm2_mdl::GetVirtAtomCount(void)
 {
 	i32s virt_atom_count = 0;
@@ -459,7 +532,10 @@ void mm2_mdl::UpdateConstraints(void)
 	}
 }
 
-bool mm2_mdl::ReadStream(istream & istr)
+/*##############################################*/
+/*##############################################*/
+
+bool mm2_mdl::ReadStream(istream & istr, bool selected)
 {
 	i32s ncs; istr >> buffer >> ncs;
 	PushCRDSets(ncs - cs_vector.size());
@@ -577,72 +653,19 @@ void mm2_mdl::WriteStream(ostream & ostr)
 	ostr << "#end" << endl;
 }
 
-void mm2_mdl::PushCRDSets(i32u p1)
+void mm2_mdl::ImportPDB(const char * fn, i32s chain_index)
 {
-	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());
+	mm1_mdl * mdl1 = new mm1_mdl(NULL, * console_class_factory::GetInstance());
 	
-	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
-	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
-	{
-		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
-		{
-			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
-			{
-				for (i32u n4 = 0;n4 < p1;n4++)
-				{
-					chn_vector[n1].res_vector[n2].crd_vector[n3].push_back(newcrd);
-				}
-			}
-			
-			for (i32u n3 = 0;n3 < p1;n3++)
-			{
-				chn_vector[n1].res_vector[n2].peptide_vector.push_back(0.0);
-			}
-		}
-	}
+	mm1_readpdb_mdata * mdata = mdl1->readpdb_ReadMData(fn);
+	mdl1->readpdb_ReadData(fn, mdata, chain_index);
+	Reduce(mdl1, mdata);
+	
+	delete mdl1; delete mdata;
 }
 
-void mm2_mdl::PopCRDSets(i32u)
-{
-cout << "Oops!!! This function is not yet ready." << endl;
-//exit(EXIT_FAILURE);
-}
-
-void mm2_mdl::CopyCRDSet(i32u p1, i32u p2)
-{
-	for (i32u n1 = 0;n1 < chn_vector.size();n1++)
-	{
-		for (i32u n2 = 0;n2 < chn_vector[n1].res_vector.size();n2++)
-		{
-			for (i32u n3 = 0;n3 < chn_vector[n1].res_vector[n2].natm;n3++)
-			{
-				chn_vector[n1].res_vector[n2].crd_vector[n3][p2][0] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][0];
-				chn_vector[n1].res_vector[n2].crd_vector[n3][p2][1] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][1];
-				chn_vector[n1].res_vector[n2].crd_vector[n3][p2][2] = chn_vector[n1].res_vector[n2].crd_vector[n3][p1][2];
-			}
-			
-			chn_vector[n1].res_vector[n2].peptide_vector[p2] = chn_vector[n1].res_vector[n2].peptide_vector[p1];
-		}
-	}
-}
-
-void mm2_mdl::SwapCRDSets(i32u, i32u)
-{
-cout << "Oops!!! This function is not yet ready." << endl;
-//exit(EXIT_FAILURE);
-}
-
-void mm2_mdl::CenterCRDSet(i32u)
-{
-cout << "Oops!!! This function is not yet ready." << endl;
-//exit(EXIT_FAILURE);
-}
-
-void mm2_mdl::ReserveCRDSets(i32u)
-{
-cout << "Oops!!! This function is not yet ready." << endl;
-//exit(EXIT_FAILURE);
-}
+/*##############################################*/
+/*##############################################*/
 
 void mm2_mdl::DoEnergy(void)
 {
@@ -681,17 +704,37 @@ void mm2_mdl::DoGeomOpt(mm2_geomopt_param & param)
 	mm2_eng * eng = new mm2_eng(* this); CopyCRD(this, eng, 0);
 	mm2_geomopt * opt = new mm2_geomopt(eng, 50, 0.025);	// optimal settings?!?!?
 	
-	for (i32s n1 = 0;n1 < param.nsteps;n1++)
+	i32s n1 = 0;	// n1 counts the number of steps...
+	while (true)
 	{
 		opt->TakeCGStep(conjugate_gradient::Newton2An);
 		sprintf(buffer, "%d %10.3f  %4.6e", n1, opt->optval, opt->optstp);
 		cout << buffer << endl;
 		
-		if (!(n1 % 10) || (n1 + 1) == param.nsteps)
+		bool terminate = false;
+		
+		if (param.enable_nsteps)	// the nsteps test...
+		{
+			if (n1 >= param.treshold_nsteps)
+			{
+				terminate = true;
+				cout << "the nsteps termination test was passed." << endl;
+			}
+		}
+		
+		// no other tests implemented yet...
+		// no other tests implemented yet...
+		// no other tests implemented yet...
+		
+		if (!(n1 % 10) || terminate)
 		{
 			CopyCRD(eng, this, 0); CenterCRDSet(0);
 			UpdateAllGraphicsViews(true);
 		}
+		
+		if (terminate) break;		// exit the loop here!!!
+		
+		n1++;	// update the number of steps...
 	}
 	
 	delete opt;
@@ -806,6 +849,9 @@ void mm2_mdl::MolDynGetParam(mm2_moldyn_param & param)
 	param.confirm = true;
 }
 
+/*##############################################*/
+/*##############################################*/
+
 void mm2_mdl::OpenTrajectory(const char * fn)
 {
 	if (!trajfile)
@@ -868,6 +914,7 @@ void mm2_mdl::ReadFrame(void)
 	}
 }
 
+/*################################################################################################*/
 /*################################################################################################*/
 
 // f = a/r
@@ -1077,6 +1124,7 @@ fGL mm2_GetSASValue(fGL * crd, mm2_mdl * ref, fGL * grad)
 	return sasv;
 }
 
+/*################################################################################################*/
 /*################################################################################################*/
 
 mm2_superimpose::mm2_superimpose(mm2_mdl * p1, i32s p2, i32s p3) : superimpose(p2, p3)
