@@ -40,8 +40,13 @@ class qm2_mdl : virtual public model_simple
 	
 	const char * GetProjectFileNameExtension(void);		// virtual
 	
-	bool ReadStream(istream &) { return false; }	// virtual
-	void WriteStream(ostream &) { }			// virtual
+	bool ReadStream(istream &, bool = false) { return false; }	// virtual
+	void WriteStream(ostream &) { }					// virtual
+	
+	/** someday we might move ImportFile() to all_atoms_interface,
+	    but at the moment we just have a dummy function here...
+	*/
+	bool ImportFile(const char*, int) { return false; }	// virtual
 };
 
 /*################################################################################################*/
