@@ -42,6 +42,25 @@ camera::~camera(void)
 {
 }
 
+bool camera::CopySettings(const camera * p1)
+{
+	ol_static * ref = dynamic_cast<ol_static *>(ol);
+	if (ref == NULL) return false;
+	
+	// now that we have verified that we can modify the ol-object, we will copy the settings...
+	
+	focus = p1->focus;
+	for (i32s n1 = 0;n1 < 3;n1++) GetLocDataRW()->crd[n1] = p1->GetLocData()->crd[n1];
+	GetLocDataRW()->zdir = p1->GetLocData()->zdir;
+	GetLocDataRW()->ydir = p1->GetLocData()->ydir;
+	
+	// the local lights that are attached to the camera are not moved?!?!?!
+	// the local lights that are attached to the camera are not moved?!?!?!
+	// the local lights that are attached to the camera are not moved?!?!?!
+	
+	return true;
+}
+
 // when rotating or translating a camera we must make all operations in reversed directions
 // and also drag all those local lights along...
 
@@ -483,27 +502,21 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 	if (gv->draw_info && rm == Normal)
 	{
 		glColor3f(1.0, 1.0, 1.0);
-		glDisable(GL_DEPTH_TEST);
-		
-		glPushMatrix();
-		glLoadIdentity();
-		
-		glMatrixMode(GL_PROJECTION);
-		glPushMatrix(); glLoadIdentity();
-		gluOrtho2D(0, gv->size[0], 0, gv->size[1]);
 		
 		// print out the selected object, if there is any...
 		// print out the selected object, if there is any...
 		// print out the selected object, if there is any...
 		
+		fGL xpos; fGL ypos;
 		void * font = GLUT_BITMAP_TIMES_ROMAN_10;
-		const char * txt = "[no selected object]"; fGL xpos; fGL ypos;
+		
+		const char * txt = "[no selected object]";
 		if (gv->GetDV()->selected_object) txt = gv->GetDV()->selected_object->GetObjectName();
 		
-		xpos = (gv->size[0] - gv->GetBitmapStringWidth(font, txt)) / 2.0;
+		xpos = (gv->size[0] - gv->GetGlutStringWidth(txt, font)) / 2.0;
 		ypos = 5.0;
 		
-		glRasterPos2f(xpos, ypos); gv->WriteBitmapString(font, txt);
+		gv->WriteGlutString2D(txt, xpos, ypos, font);
 		
 		// print out the current tool...
 		// print out the current tool...
@@ -511,20 +524,10 @@ void camera::RenderWindow(graphics_view * gv, rmode rm, int x, int y)
 		
 		txt = gv->current_tool->GetToolName();
 		
-		xpos = (gv->size[0] - gv->GetBitmapStringWidth(font, txt)) / 2.0;
+		xpos = (gv->size[0] - gv->GetGlutStringWidth(txt, font)) / 2.0;
 		ypos = gv->size[1] - 10.0;
 		
-		glRasterPos2f(xpos, ypos); gv->WriteBitmapString(font, txt);
-		
-		// ok...
-		// ok...
-		// ok...
-		
-		glPopMatrix();
-		glMatrixMode(GL_MODELVIEW);
-		
-		glPopMatrix();
-		glEnable(GL_DEPTH_TEST);
+		gv->WriteGlutString2D(txt, xpos, ypos, font);
 	}
 	
 #endif	// TARGET2
