@@ -12,6 +12,10 @@
 #ifdef ENABLE_GRAPHICS
 #include "views.h"
 
+#include "v3d.h"
+#include "Vector.h"	// OpenBabel utils for torsion, angle measure
+using namespace OpenBabel;
+
 #include <iomanip>
 #include <strstream>
 #include <algorithm>
@@ -477,15 +481,43 @@ bool docview::RemoveObject(dummy_object * p1)
 	delete p1; return true;
 }
 
+// these are the measuring functions, that only take coordinates as input (so they are model-independent)...
+// these are the measuring functions, that only take coordinates as input (so they are model-independent)...
+// these are the measuring functions, that only take coordinates as input (so they are model-independent)...
+
+float measure_len(float * c1, float * c2)
+{
+	v3d<float> v1(c1, c2);
+	return v1.len();
+}
+
+float measure_ang(float * c1, float * c2, float * c3)
+{
+	Vector v1, v2;
+	v1 = Vector(c1[0] - c2[0], c1[1] - c2[1], c1[2] - c2[2]);
+	v2 = Vector(c3[0] - c2[0], c3[1] - c2[1], c3[2] - c2[2]);
+	return VectorAngle(v1, v2);
+}
+
+float measure_tor(float * c1, float * c2, float * c3, float * c4)
+{
+	Vector v1, v2, v3, v4;
+	v1 = Vector(c1[0], c1[1], c1[2]) * 10.0f;
+	v2 = Vector(c2[0], c2[1], c2[2]) * 10.0f;
+	v3 = Vector(c3[0], c3[1], c3[2]) * 10.0f;
+	v4 = Vector(c4[0], c4[1], c4[2]) * 10.0f;
+	return CalcTorsionAngle(v1, v2, v3, v4);
+}
+
 /*##############################################*/
 /*##############################################*/
 
-graphics_view * docview::AddGraphicsView(void)
+graphics_view * docview::AddGraphicsView(bool detached)
 {
 	fGL focus = GetDefaultFocus();
 	camera_vector.push_back(new camera(ol_static(), focus, this));
 	
-	graphics_view_vector.push_back(graphics_factory->ProduceGraphicsView(this, camera_vector.back()));
+	graphics_view_vector.push_back(graphics_factory->ProduceGraphicsView(this, camera_vector.back(), detached));
 	
 	// also add a default light, so that we can see something...
 	// also add a default light, so that we can see something...
@@ -497,9 +529,11 @@ graphics_view * docview::AddGraphicsView(void)
 	return graphics_view_vector.back();
 }
 
-graphics_view * docview::AddGraphicsView(camera * p1)
+// here the camera pointer "p1" should already be in the camera_vector; see the above case!!!
+
+graphics_view * docview::AddGraphicsView(camera * p1, bool detached)
 {
-	graphics_view_vector.push_back(graphics_factory->ProduceGraphicsView(this, p1));
+	graphics_view_vector.push_back(graphics_factory->ProduceGraphicsView(this, p1, detached));
 	
 	// setup the old lights that we already have...
 	// setup the old lights that we already have...
@@ -697,6 +731,19 @@ void docview::UpdateGraphicsView(graphics_view * gv, bool flag)
 	gv->Update(flag);
 }
 
+void docview::UpdatePlottingViews(void)
+{
+	for (i32u n1 = 0;n1 < plotting_view_vector.size();n1++)
+	{
+		// energy-level diagrams might require scaling here?!?!?!
+		// detect them and call the scaling function... ANY BETTER WAY TO DO THIS???
+		enlevdiag_view * eldv = dynamic_cast<enlevdiag_view *>(plotting_view_vector[n1]);
+		if (eldv != NULL) eldv->SetCenterAndScale();
+		
+		plotting_view_vector[n1]->Update();
+	}
+}
+
 void docview::RenderAllTPs(graphics_view * gv, rmode)
 {
 	// here we will render those transparent primitives...
@@ -767,25 +814,33 @@ void docview::InitGL(void)
         const fGL default_background[4] = { 0.0, 0.0, 0.0, 1.0};
 	fGL *background = model_prefs->ColorRGBA("Graphics/BkgndColor", default_background);
 	glClearColor(background[0], background[1], background[2], background[3]);
-	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
+	delete [] background;
 	
+	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
 	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
+	
 	const fGL default_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
 	fGL *specular_reflectance = model_prefs->ColorRGBA("Graphics/SpecularReflectance", default_reflectance);
 	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
+	delete [] specular_reflectance;
+	
 	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
 	glEnable(GL_COLOR_MATERIAL);
 	
 	const fGL default_ambient[4] = { 0.2, 0.2, 0.2, 1.0 };
 	fGL *ambient_intensity = model_prefs->ColorRGBA("Graphics/AmbientIntensity", default_ambient);
 	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
+	delete [] ambient_intensity;
+	
 	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
 	
-	glFogi(GL_FOG_MODE, GL_EXP); 
+	glFogi(GL_FOG_MODE, GL_EXP);
 	glFogf(GL_FOG_DENSITY, model_prefs->Double("Graphics/FogDensity",0.15));
+	
 	const fGL default_fog[4] = { 0.0, 0.0, 0.0, 0.0 };
 	fGL *fog_color = model_prefs->ColorRGBA("Graphics/FogColor", default_fog);
 	glFogfv(GL_FOG_COLOR, fog_color);
+	delete [] fog_color;
 	
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 	
