@@ -22,8 +22,6 @@
 #include "color.h"
 #include "views.h"
 
-#include "Vector.h" // OELib utils for torsion, angle measure
-
 #include <fstream>
 #include <strstream>
 using namespace std;
@@ -89,6 +87,20 @@ void mm1_docv::InvertSelection(void)
 	UpdateAllGraphicsViews();
 }
 
+//void mm1_docv::SelectNone(void)
+//{
+//	if (selected_object != NULL)
+//	{
+//		selected_object = NULL;
+//		event_SelectedObjectChanged();
+//	}
+//	
+//	iter_mm1al it1 = atom_list.begin();
+//	while (it1 != atom_list.end()) (* it1++).selected = false;
+//	
+//	UpdateAllGraphicsViews();
+//}
+
 bool mm1_docv::TestAtom(mm1_atom * ref, rmode rm)
 {
 	// what about the "visible"-flag???
@@ -116,7 +128,7 @@ void mm1_docv::SetColor(color_mode * cm, mm1_atom * ref)
 {
 	const fGL default_sel_color[3] = { 1.0, 0.0, 1.0 };
 	fGL *select_color = model_prefs->ColorRGB("Graphics/SelectColor", default_sel_color);
-  
+
 	if (ref->selected) glColor3f(select_color[0], select_color[1], select_color[2]);
 	else
 	{
@@ -124,6 +136,8 @@ void mm1_docv::SetColor(color_mode * cm, mm1_atom * ref)
 		cm->GetColor(ref, color, model_prefs);
 		glColor3fv(color);
 	}
+
+	delete [] select_color;
 }
 
 // it seems that there is something common for all models (like the sel-buffer) -> move to the docview?!?!
@@ -132,11 +146,10 @@ void mm1_docv::SetColor(color_mode * cm, mm1_atom * ref)
 
 void mm1_docv::Render(graphics_view * gv, rmode rm)
 {
-	int tmp;
-	const fGL default_label_color[3] = { 1.0, 1.0, 1.0 };
+	const fGL default_label_color[3] = { 0.0, 1.0, 1.0 };	// looks bad but won't fade easily into other colours...
 	fGL *label_color = model_prefs->ColorRGB("Graphics/LabelColor", default_label_color);
 	
-bool accum = gv->accumulate; if (rm != Normal) accum = false;
+	bool accum = gv->accumulate; if (rm != Normal) accum = false;
 //if (accum) { glClear(GL_ACCUM_BUFFER_BIT); UpdateAccumValues(); }
 //else if (rm != Transform2) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
@@ -214,54 +227,90 @@ bool accum = gv->accumulate; if (rm != Normal) accum = false;
 	if (accum) glAccum(GL_RETURN, 1.0);
 	else if (rm != Transform2) gv->cam->RenderObjects(gv);
 	
-	if (gv->label == LABEL_ELEMENT)
+	if (gv->label == LABEL_INDEX)
 	{
-		glDisable(GL_DEPTH_TEST);  char string[32]; 
+		char string[32]; i32s tmp1 = 0;
+		
 		glColor3f(label_color[0], label_color[1], label_color[2]);
 		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
 		{
-			glRasterPos3fv((* it1).crd_vector[0].data);
+			ostrstream str(string, sizeof(string));
+			str << tmp1++ << ends;
 			
+			fGL x = (* it1).crd_vector[0].data[0];
+			fGL y = (* it1).crd_vector[0].data[1];
+			fGL z = (* it1).crd_vector[0].data[2];
+			
+			gv->WriteGlutString3D(string, x, y, z, gv->cam->GetLocData(), GLUT_BITMAP_9_BY_15);
+		}
+	}
+	else if (gv->label == LABEL_CHARGE)
+	{
+		char string[32];
+		
+		glColor3f(label_color[0], label_color[1], label_color[2]);
+		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
+		{
+			ostrstream str(string, sizeof(string));
+			str.setf(ios::fixed | ios::showpos); str.precision(4); str << (* it1).charge << ends;
+
+			fGL x = (* it1).crd_vector[0].data[0];
+			fGL y = (* it1).crd_vector[0].data[1];
+			fGL z = (* it1).crd_vector[0].data[2];
+			
+			gv->WriteGlutString3D(string, x, y, z, gv->cam->GetLocData(), GLUT_BITMAP_9_BY_15);
+		}
+	}
+	else if (gv->label == LABEL_ELEMENT)
+	{
+		char string[32];
+		
+		glColor3f(label_color[0], label_color[1], label_color[2]);
+		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
+		{
 			ostrstream str(string, sizeof(string));
 			str << (* it1).el.GetSymbol() << ends;
-			gv->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
+
+			fGL x = (* it1).crd_vector[0].data[0];
+			fGL y = (* it1).crd_vector[0].data[1];
+			fGL z = (* it1).crd_vector[0].data[2];
+			
+			gv->WriteGlutString3D(string, x, y, z, gv->cam->GetLocData(), GLUT_BITMAP_9_BY_15);
 		}
-		glEnable(GL_DEPTH_TEST);  
 	}
-	else if (gv->label == LABEL_ATOMID)
+	else if (gv->label == LABEL_ATOMTYPE)
 	{
-		glDisable(GL_DEPTH_TEST);  char string[32]; 
+		char string[32];
+		
 		glColor3f(label_color[0], label_color[1], label_color[2]);
-		tmp = 0;
 		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
 		{
-			glRasterPos3fv((* it1).crd_vector[0].data);
-			
 			ostrstream str(string, sizeof(string));
-			str << tmp++ << ends;
-			gv->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
+			str << "0x" << hex << (* it1).atmtp << ends;
+
+			fGL x = (* it1).crd_vector[0].data[0];
+			fGL y = (* it1).crd_vector[0].data[1];
+			fGL z = (* it1).crd_vector[0].data[2];
+			
+			gv->WriteGlutString3D(string, x, y, z, gv->cam->GetLocData(), GLUT_BITMAP_9_BY_15);
 		}
-		glEnable(GL_DEPTH_TEST);  
 	}
 	else if (gv->label == LABEL_BONDTYPE)
 	{
-		glDisable(GL_DEPTH_TEST);  char string[32]; 
+		char string[32];
+		
 		glColor3f(label_color[0], label_color[1], label_color[2]);
 		for (iter_mm1bl it1 = bond_list.begin();it1 != bond_list.end();it1++)
 		{
-			fGL rpos[3];
-			for (i32s n1 = 0;n1 < 3;n1++)
-			{
-				fGL tmp1 = (* it1).atmr[0]->crd_vector[0][n1];
-				fGL tmp2 = (* it1).atmr[1]->crd_vector[0][n1];
-				rpos[n1] = (tmp1 + tmp2) / 2.0;
-			} glRasterPos3fv(rpos);
-			
 			ostrstream str(string, sizeof(string));
 			str << (* it1).bt.GetSymbol1() << ends;
-			gv->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
+
+			fGL x = ((* it1).atmr[0]->crd_vector[0][0] + (* it1).atmr[1]->crd_vector[0][0]) / 2.0;
+			fGL y = ((* it1).atmr[0]->crd_vector[0][1] + (* it1).atmr[1]->crd_vector[0][1]) / 2.0;
+			fGL z = ((* it1).atmr[0]->crd_vector[0][2] + (* it1).atmr[1]->crd_vector[0][2]) / 2.0;
+			
+			gv->WriteGlutString3D(string, x, y, z, gv->cam->GetLocData(), GLUT_BITMAP_9_BY_15);
 		}
-		glEnable(GL_DEPTH_TEST);  
 	}
 	
 	if (gv->enable_fog) glDisable(GL_FOG);
@@ -271,6 +320,8 @@ bool accum = gv->accumulate; if (rm != Normal) accum = false;
 	// finally call this to handle transparency...
 	
 	RenderAllTPs(gv, rm);
+
+	delete [] label_color;
 }
 
 void mm1_docv::RenderScene(graphics_view * gv, rmode rm, bool accum)
@@ -686,6 +737,7 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
         static mm1_atom *a1 = NULL;
 	static mm1_atom *a2 = NULL;
 	static mm1_atom *a3 = NULL;
+// this function is quite similar in mm1_docv and qm1_docv. could this be shorter as a loop???
 
 	if (names[0] == GLNAME_MD_TYPE1)
 	{
@@ -700,86 +752,406 @@ void mm1_docv::MeasureEvent(graphics_view *, vector<iGLu> & names)
 		  }
 		else if (a1 != NULL && a2 == NULL)
 		  {
-		    if (a1 == ref)
-		      {
-			a1->selected = false; a1 = NULL;
-			return;
-		      }
-
+		    if (a1 == ref) { a1->selected = false; a1 = NULL; return; }
+		    
 		    a2 = ref;
+		    
 		    float * p1 = a1->crd_vector[0].data;
 		    float * p2 = a2->crd_vector[0].data;
-		    v3d<float> v1(p1, p2);
-		    cout << "distance: " << v1.len() << " nm" << endl;
+		    
+		    float len = measure_len(p1, p2);
+		    cout << "distance: " << len << " nm" << endl;
 		  }
 		else if (a1 != NULL && a2 != NULL && a3 == NULL)
 		  {
-		    if (a1 == ref)
-		      {
-			a1->selected = false; a1 = a2; a2 = NULL;
-			return;
-		      }
-		    else if (a2 == ref)
-		      {
-			a2->selected = false; a2 = NULL;
-			return;
-		      }
-
-		    Vector v1, v2;
-		    v1 = Vector(a1->crd_vector[0][0] - a2->crd_vector[0][0],
-				a1->crd_vector[0][1] - a2->crd_vector[0][1],
-				a1->crd_vector[0][2] - a2->crd_vector[0][2]);
-		    v2 = Vector(ref->crd_vector[0][0] - a2->crd_vector[0][0],
-				ref->crd_vector[0][1] - a2->crd_vector[0][1],
-				ref->crd_vector[0][2] - a2->crd_vector[0][2]);
-
+		    if (a1 == ref) { a1->selected = false; a1 = a2; a2 = NULL; return; }
+		    else if (a2 == ref) { a2->selected = false; a2 = NULL; return; }
+		    
 		    a3 = ref;
-		    cout << "angle: " << VectorAngle(v1, v2) << endl;
+		    
+		    float * p1 = a1->crd_vector[0].data;
+		    float * p2 = a2->crd_vector[0].data;
+		    float * p3 = a3->crd_vector[0].data;
+		    
+		    float ang = measure_ang(p1, p2, p3);
+		    cout << "angle: " << ang << " deg" << endl;
 		  }
 		else
 		  {
-		    if (a1 == ref)
-		      {
-			a1->selected = false; a1 = a2; a2 = a3; a3 = NULL;
-			return;
-		      }
-		    else if (a2 == ref)
-		      {
-			a2->selected = false; a2 = a3; a3 = NULL;
-			return;
-		      }
-		    else if (a3 == ref)
-		      {
-			a3->selected = false; a3 = NULL;
-			return;
-		      }
-		    Vector v1, v2, v3, v4;
-		    v1 = Vector(a1->crd_vector[0][0],
-				a1->crd_vector[0][1],
-				a1->crd_vector[0][2]) * 10.0f;
-		    v2 = Vector(a2->crd_vector[0][0],
-				a2->crd_vector[0][1],
-				a2->crd_vector[0][2]) * 10.0f;
-		    v3 = Vector(a3->crd_vector[0][0],
-				a3->crd_vector[0][1],
-				a3->crd_vector[0][2]) * 10.0f;
-		    v4 = Vector(ref->crd_vector[0][0],
-				ref->crd_vector[0][1],
-				ref->crd_vector[0][2]) * 10.0f;
-
-		    float tor = CalcTorsionAngle(v1, v2, v3, v4);
-		    cout << "torsion: " << tor << endl;
-
+		    if (a1 == ref) { a1->selected = false; a1 = a2; a2 = a3; a3 = NULL; return; }
+		    else if (a2 == ref) { a2->selected = false; a2 = a3; a3 = NULL; return; }
+		    else if (a3 == ref) { a3->selected = false; a3 = NULL; return; }
+		    
+		    float * p1 = a1->crd_vector[0].data;
+		    float * p2 = a2->crd_vector[0].data;
+		    float * p3 = a3->crd_vector[0].data;
+		    float * p4 = ref->crd_vector[0].data;
+		    
+		    float tor = measure_tor(p1, p2, p3, p4);
+		    cout << "torsion: " << tor << " deg " << endl;
+		    
 		    a1->selected = false; a1 = NULL;
 		    a2->selected = false; a2 = NULL;
 		    a3->selected = false; a3 = NULL;
 		    ref->selected = false;
+		    
 		    UpdateAllGraphicsViews();
 		  }
-
 	}
 }
 
+void mm1_docv::DoEnergyPlot1D(void)
+{
+	char question; i32s osteps = 100;
+	i32s qm_eng_index = NOT_DEFINED; i32s qm_total_charge = 0;
+	
+	i32s molnum = 0; i32s in_crdset = 0;
+	
+	f64 range1[2] = { 0.0, 2.0 * M_PI }; i32s steps1 = 36;
+	i32s atmi1[4]; mm1_atom * atmr1[4];
+cout << "give the atom ID numbers (separated by spaces) for torsion : ";
+cin >> atmi1[0] >> atmi1[1] >> atmi1[2] >> atmi1[3];
+cout << "do you want to change the default settings (y=yes)? "; cin >> question;
+if (question == 'y' || question == 'Y')
+{
+	cout << "give the number of steps (-1 = cancel, default = " << steps1 << ")? "; cin >> steps1;
+	if (steps1 < 0) return;
+	
+	f64 range[2] = { 180.0 * range1[0] / M_PI, 180.0 * range1[0] / M_PI };
+	cout << "give the starting angle (degrees, default = " << range[0] << ")? "; cin >> range[0];
+	cout << "give the ending angle (degrees, default = " << range[1] << ")? "; cin >> range[1];
+	
+	if (range[0] >= range[1]) { cout << "error: starting angle must be smaller than ending angle." << endl; return; }
+	if (range[1] - range[0] > 360.0) { cout << "error: the range is now more than full circle 360 degrees." << endl; return; }
+	
+	range1[0] = M_PI * range[0] / 180.0;
+	range1[1] = M_PI * range[1] / 180.0;
+}
+
+cout << "give the optimization steps (-1 to skip optimization, default = " << osteps << ")? "; cin >> osteps;
+cout << "should the final energy be calculated using a QM method (y=yes)? "; cin >> question;
+if (question == 'y' || question == 'Y')
+{
+	i32s index = 0;
+	while (true)
+	{
+		if (qm1_mdl::engtab1[index] == NULL) break;
+		
+		cout << index << " " << qm1_mdl::engtab1[index] << endl;
+		index++;
+	}
+	
+	cout << "give the method from the above list (or -1 to cancel)? "; cin >> qm_eng_index;
+	if (qm_eng_index >= index) qm_eng_index = NOT_DEFINED;	// check the index...
+	
+	cout << "give the total charge for the QM model (default = " << qm_total_charge << ")? "; cin >> qm_total_charge;
+}
+
+	for (i32s n1 = 0;n1 < 4;n1++)
+	{
+		iter_mm1al it1;
+		
+		it1 = FindAtomByIndex(atmi1[n1]);
+		if (it1 == GetAtomsEnd()) cout << "atom not found!" << endl;
+		atmr1[n1] = & (* it1);
+	}
+	
+	GatherGroups();		// for internal coordinates...
+	mm1_intcrd * ic = new mm1_intcrd((* this), molnum, in_crdset);
+	i32s ict1 = ic->FindTorsion(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
+if (ict1 < 0) { cout << "could not find ic tor1." << endl; return; }
+	
+	v3d<fGL> v1a(atmr1[1]->crd_vector[in_crdset].data, atmr1[0]->crd_vector[in_crdset].data);
+	v3d<fGL> v1b(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
+	v3d<fGL> v1c(atmr1[2]->crd_vector[in_crdset].data, atmr1[3]->crd_vector[in_crdset].data);
+	f64 oldt1 = v1a.tor(v1b, v1c);
+	
+	mm1_eng * eng1 = CreateDefaultEngine();
+	i32s fft1 = eng1->FindTorsion(atmr1[0], atmr1[1], atmr1[2], atmr1[3]);
+if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
+
+	mm1_eng * eng2 = CreateDefaultEngine();
+	
+	plot1d_view * plot = graphics_factory->ProducePlot1DView(this, PLOT_USERDATA_STRUCTURE, 0);
+	
+	f64 tor1 = range1[0];
+	for (i32s s1 = 0;s1 < (steps1 + 1);s1++)
+	{
+		ic->SetTorsion(ict1, tor1 - oldt1);
+		ic->UpdateCartesian();
+		
+		CopyCRD(this, eng1, 0);		// lock_local_structure needs coordinates!!!
+		eng1->SetTorsionConstraint(fft1, tor1, 5000.0, true);
+		
+		// optimize...
+		
+		mm1_geomopt * opt = new mm1_geomopt(eng1, 100, 0.025);		// optimal settings?!?!?
+		
+		for (i32s n1 = 0;n1 < osteps;n1++)
+		{
+			opt->TakeCGStep(conjugate_gradient::Newton2An);
+			cout << n1 << " " << opt->optval << " " << opt->optstp << endl;
+			
+			if (!(n1 % 10))
+			{
+				CopyCRD(eng1, this, 0); CenterCRDSet(0);
+				UpdateAllGraphicsViews(true);
+			}
+		}
+		
+		CopyCRD(eng1, this, 0); CenterCRDSet(0);
+		delete opt;
+		
+		// compute energy for final structure...
+		
+		f64 value;
+		if (qm_eng_index < 0)
+		{
+			CopyCRD(this, eng2, 0);
+			eng2->Compute(0);
+			
+			value = eng2->energy;
+		}
+		else
+		{
+			qm1_mdl * mdl9 = new qm1_mdl(& cout, * console_class_factory::GetInstance());
+			mdl9->aai_MakeCopy(this);
+			
+			mdl9->default_eng = qm_eng_index;
+			mdl9->SetTotalCharge(qm_total_charge);
+			qm1_eng * eng9 = mdl9->CreateDefaultEngine();
+			
+			if (eng9 != NULL)
+			{
+				CopyCRD(mdl9, eng9, 0);
+				eng9->Compute(0);
+				
+				value = eng9->energy;
+				
+				delete eng9;
+				delete mdl9;
+			}
+			else
+			{
+				value = 0.0;	// if something went wrong...
+				delete mdl9;	// ...we just use zero!!!
+			}
+		}
+		
+		// ...and add it to the plot.
+		
+		void * udata = mm1_convert_cset_to_plotting_udata(this, 0);
+		plot->AddData(180.0 * tor1 / M_PI, value, udata);
+		
+		tor1 += (range1[1] - range1[0]) / (f64) steps1;
+	}
+	
+	plotting_view_vector.push_back(plot);
+	plot->SetCenterAndScale();
+	plot->Update();
+	
+	delete eng2;
+	delete eng1;
+	delete ic;
+}
+
+void mm1_docv::DoEnergyPlot2D(void)
+{
+	char question; i32s osteps = 100;
+	i32s qm_eng_index = NOT_DEFINED; i32s qm_total_charge = 0;
+	
+	i32s molnum = 0; i32s in_crdset = 0;
+	
+	f64 range1[2] = { 0.0, 2.0 * M_PI }; i32s steps1 = 18;
+	i32s atmi1[4]; mm1_atom * atmr1[4];
+cout << "give the atom ID numbers (separated by spaces) for torsion1 : ";
+cin >> atmi1[0] >> atmi1[1] >> atmi1[2] >> atmi1[3];
+cout << "do you want to change the default settings (y=yes)? "; cin >> question;
+if (question == 'y' || question == 'Y')
+{
+	cout << "give the number of steps (-1 = cancel, default = " << steps1 << ")? "; cin >> steps1;
+	if (steps1 < 0) return;
+	
+	f64 range[2] = { 180.0 * range1[0] / M_PI, 180.0 * range1[0] / M_PI };
+	cout << "give the starting angle (degrees, default = " << range[0] << ")? "; cin >> range[0];
+	cout << "give the ending angle (degrees, default = " << range[1] << ")? "; cin >> range[1];
+	
+	if (range[0] >= range[1]) { cout << "error: starting angle must be smaller than ending angle." << endl; return; }
+	if (range[1] - range[0] > 360.0) { cout << "error: the range is now more than full circle 360 degrees." << endl; return; }
+	
+	range1[0] = M_PI * range[0] / 180.0;
+	range1[1] = M_PI * range[1] / 180.0;
+}
+
+	f64 range2[2] = { 0.0, 2.0 * M_PI }; i32s steps2 = 18;
+	i32s atmi2[4]; mm1_atom * atmr2[4];
+cout << "give the atom ID numbers (separated by spaces) for torsion2 : ";
+cin >> atmi2[0] >> atmi2[1] >> atmi2[2] >> atmi2[3];
+cout << "do you want to change the default settings (y=yes)? "; cin >> question;
+if (question == 'y' || question == 'Y')
+{
+	cout << "give the number of steps (-1 = cancel, default = " << steps2 << ")? "; cin >> steps2;
+	if (steps2 < 0) return;
+	
+	f64 range[2] = { 180.0 * range2[0] / M_PI, 180.0 * range2[0] / M_PI };
+	cout << "give the starting angle (degrees, default = " << range[0] << ")? "; cin >> range[0];
+	cout << "give the ending angle (degrees, default = " << range[1] << ")? "; cin >> range[1];
+	
+	if (range[0] >= range[1]) { cout << "error: starting angle must be smaller than ending angle." << endl; return; }
+	if (range[1] - range[0] > 360.0) { cout << "error: the range is now more than full circle 360 degrees." << endl; return; }
+	
+	range2[0] = M_PI * range[0] / 180.0;
+	range2[1] = M_PI * range[1] / 180.0;
+}
+
+cout << "give the optimization steps (-1 to skip optimization, default = " << osteps << ")? "; cin >> osteps;
+cout << "should the final energy be calculated using a QM method (y=yes)? "; cin >> question;
+if (question == 'y' || question == 'Y')
+{
+	i32s index = 0;
+	while (true)
+	{
+		if (qm1_mdl::engtab1[index] == NULL) break;
+		
+		cout << index << " " << qm1_mdl::engtab1[index] << endl;
+		index++;
+	}
+	
+	cout << "give the method from the above list (or -1 to cancel)? "; cin >> qm_eng_index;
+	if (qm_eng_index >= index) qm_eng_index = NOT_DEFINED;	// check the index...
+	
+	cout << "give the total charge for the QM model (default = " << qm_total_charge << ")? "; cin >> qm_total_charge;
+}
+
+	for (i32s n1 = 0;n1 < 4;n1++)
+	{
+		iter_mm1al it1;
+		
+		it1 = FindAtomByIndex(atmi1[n1]);
+		if (it1 == GetAtomsEnd()) cout << "atom not found #1!" << endl;
+		atmr1[n1] = & (* it1);
+		
+		it1 = FindAtomByIndex(atmi2[n1]);
+		if (it1 == GetAtomsEnd()) cout << "atom not found #2!" << endl;
+		atmr2[n1] = & (* it1);
+	}
+	
+	GatherGroups();		// for internal coordinates...
+	mm1_intcrd * ic = new mm1_intcrd((* this), molnum, in_crdset);
+	i32s ict1 = ic->FindTorsion(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
+if (ict1 < 0) { cout << "could not find ic tor1." << endl; return; }
+	i32s ict2 = ic->FindTorsion(atmr2[1]->crd_vector[in_crdset].data, atmr2[2]->crd_vector[in_crdset].data);
+if (ict2 < 0) { cout << "could not find ic tor2." << endl; return; }
+	
+	v3d<fGL> v1a(atmr1[1]->crd_vector[in_crdset].data, atmr1[0]->crd_vector[in_crdset].data);
+	v3d<fGL> v1b(atmr1[1]->crd_vector[in_crdset].data, atmr1[2]->crd_vector[in_crdset].data);
+	v3d<fGL> v1c(atmr1[2]->crd_vector[in_crdset].data, atmr1[3]->crd_vector[in_crdset].data);
+	f64 oldt1 = v1a.tor(v1b, v1c);
+	
+	v3d<fGL> v2a(atmr2[1]->crd_vector[in_crdset].data, atmr2[0]->crd_vector[in_crdset].data);
+	v3d<fGL> v2b(atmr2[1]->crd_vector[in_crdset].data, atmr2[2]->crd_vector[in_crdset].data);
+	v3d<fGL> v2c(atmr2[2]->crd_vector[in_crdset].data, atmr2[3]->crd_vector[in_crdset].data);
+	f64 oldt2 = v2a.tor(v2b, v2c);
+	
+	mm1_eng * eng1 = CreateDefaultEngine();
+	i32s fft1 = eng1->FindTorsion(atmr1[0], atmr1[1], atmr1[2], atmr1[3]);
+if (ict1 < 0) { cout << "could not find ff tor1." << endl; return; }
+	i32s fft2 = eng1->FindTorsion(atmr2[0], atmr2[1], atmr2[2], atmr2[3]);
+if (ict2 < 0) { cout << "could not find ff tor2." << endl; return; }
+
+	mm1_eng * eng2 = CreateDefaultEngine();
+	
+	plot2d_view * plot = graphics_factory->ProducePlot2DView(this, PLOT_USERDATA_STRUCTURE, 0);
+	
+	f64 tor1 = range1[0];
+	for (i32s s1 = 0;s1 < (steps1 + 1);s1++)
+	{
+		f64 tor2 = range2[0];
+		for (i32s s2 = 0;s2 < (steps2 + 1);s2++)
+		{
+			ic->SetTorsion(ict1, tor1 - oldt1);
+			ic->SetTorsion(ict2, tor2 - oldt2);
+			ic->UpdateCartesian();
+			
+			CopyCRD(this, eng1, 0);		// lock_local_structure needs coordinates!!!
+			eng1->SetTorsionConstraint(fft1, tor1, 5000.0, true);
+			eng1->SetTorsionConstraint(fft2, tor2, 5000.0, true);
+			
+			// optimize...
+			
+			mm1_geomopt * opt = new mm1_geomopt(eng1, 100, 0.025);		// optimal settings?!?!?
+			
+			for (i32s n1 = 0;n1 < osteps;n1++)
+			{
+				opt->TakeCGStep(conjugate_gradient::Newton2An);
+				cout << n1 << " " << opt->optval << " " << opt->optstp << endl;
+				
+				if (!(n1 % 10))
+				{
+					CopyCRD(eng1, this, 0); CenterCRDSet(0);
+					UpdateAllGraphicsViews(true);
+				}
+			}
+			
+			CopyCRD(eng1, this, 0); CenterCRDSet(0);
+			delete opt;
+			
+			// compute energy for final structure...
+			
+			f64 value;
+			if (qm_eng_index < 0)
+			{
+				CopyCRD(this, eng2, 0);
+				eng2->Compute(0);
+				
+				value = eng2->energy;
+			}
+			else
+			{
+				qm1_mdl * mdl9 = new qm1_mdl(& cout, * console_class_factory::GetInstance());
+				mdl9->aai_MakeCopy(this);
+				
+				mdl9->default_eng = qm_eng_index;
+				mdl9->SetTotalCharge(qm_total_charge);
+				qm1_eng * eng9 = mdl9->CreateDefaultEngine();
+				
+				if (eng9 != NULL)
+				{
+					CopyCRD(mdl9, eng9, 0);
+					eng9->Compute(0);
+					
+					value = eng9->energy;
+					
+					delete eng9;
+					delete mdl9;
+				}
+				else
+				{
+					value = 0.0;	// if something went wrong...
+					delete mdl9;	// ...we just use zero!!!
+				}
+			}
+			
+			// ...and add it to the plot.
+			
+			void * udata = mm1_convert_cset_to_plotting_udata(this, 0);
+			plot->AddData(180.0 * tor1 / M_PI, 180.0 * tor2 / M_PI, value, udata);
+			
+			tor2 += (range2[1] - range2[0]) / (f64) steps2;
+		}
+		
+		tor1 += (range1[1] - range1[0]) / (f64) steps1;
+	}
+	
+	plotting_view_vector.push_back(plot);
+	plot->SetCenterAndScale();
+	plot->Update();
+	
+	delete eng2;
+	delete eng1;
+	delete ic;
+}
+
 /*################################################################################################*/
 
 void mm1_cm_element::GetColor(const void * p1, fGL_a4 & p2, prefs *p3)
@@ -789,8 +1161,6 @@ void mm1_cm_element::GetColor(const void * p1, fGL_a4 & p2, prefs *p3)
 	p2[0] = color[0]; p2[1] = color[1]; p2[2] = color[2];
 }
 
-/*################################################################################################*/
-
 void mm1_cm_state::GetColor(const void * p1, fGL_a4 & p2, prefs *p3)
 {
 //	mm1_atom * ref = (mm1_atom *) p1;
@@ -815,7 +1185,7 @@ void mm1_cm_state::GetColor(const void * p1, fGL_a4 & p2, prefs *p3)
 		case 'S':
 		p2[0] = 0.0; p2[1] = 1.0; p2[2] = 0.0;		// strand
 		return;
-	}*/
+	}	*/
 }
 
 /*################################################################################################*/
