@@ -14,6 +14,9 @@
 #include "notice.h"
 
 #include <strstream>
+//#include <popt.h>		// For parsing command-line
+#include <popt-gnome.h>			// For parsing command-line (Debian 2.2r3???)
+//#include <libgnome/gnome-popt.h>	// For parsing command-line (Debian 2.2r3???)
 using namespace std;
 
 /*################################################################################################*/
@@ -29,10 +32,10 @@ GnomeUIInfo gnome_mdi_app::file_new_submenu[] =
 	
 	GNOMEUIINFO_MENU_NEW_ITEM("mm1gp - molecular mechanics", NULL,
 		gnome_mdi_app::menu_FileNewMM1, NULL),
-		
-//	GNOMEUIINFO_MENU_NEW_ITEM("mm2gp - reduced protein model", NULL,
-//		gnome_mdi_app::menu_DummyEvent, NULL),
-		
+	
+	GNOMEUIINFO_MENU_NEW_ITEM("mm2gp - reduced protein model", NULL,
+		gnome_mdi_app::menu_FileNewMM2, NULL),
+	
 	GNOMEUIINFO_END
 };
 
@@ -328,13 +331,24 @@ void gnome_mdi_app::menu_FileNewMM1(GtkWidget *, gpointer)
 	docv->InitGnomeMDIChild();
 }
 
+void gnome_mdi_app::menu_FileNewMM2(GtkWidget *, gpointer)
+{
+	gnome_docv * docv;
+	docv = new gnome_mm2_docv(& cout, * gnome_class_factory::GetInstance(), NULL);
+	docv->InitGnomeMDIChild();
+}
+
 void gnome_mdi_app::menu_FileOpenQM1(GtkWidget *, gpointer)
 {
-	err_util * err = gnome_class_factory::GetInstance()->ProduceErrUtil();
-	err->Message("Sorry, this feature is not yet implemented.\nPlease create a new qm1gp-file instead\nand use tool \"Import mm1gp\".");
-	delete err;
-	
-//	new qm1_file_open_dialog(NULL);		// memory leak???
+err_util * err = gnome_class_factory::GetInstance()->ProduceErrUtil();
+err->Message("Please remember, that the preferred way of using QM models\nis to start with MM model and then convert it to QM model!");
+delete err;
+
+menu_FileNewQM1(NULL, NULL);	// just open an empty file, until we have specified the file format here...
+
+	// the above message is not displayed???
+	// this is not very critical message, but perhaps the err_util dialogs should be forced to modal ones?!?!?!
+//	new qm1_file_open_dialog(NULL);		// memory leak???	THIS IS NOT YET READY...
 }
 
 void gnome_mdi_app::menu_FileOpenMM1(GtkWidget *, gpointer)
@@ -671,11 +685,64 @@ void gnome_mdi_app::DestroyHandler(GtkObject *, gpointer)
 /*################################################################################################*/
 
 singleton_cleaner<gnome_mdi_app> app_cleaner;
+char *filename = NULL;
+char *importname = NULL;
 
 int main(int argc, char ** argv)
 {
-	gnome_init("ghemical", VERSION, argc, argv);
+        char **args;
+        poptContext options_context;
+        const struct poptOption options_table[] = {
+	  {"file",'f', POPT_ARG_STRING, &filename, 0, "open file", "filename"},
+	  {"import",'i', POPT_ARG_STRING, &importname, 0, "import file as mm1gp", "filename"},
+	  {NULL,'\0', 0, NULL, 0, NULL, NULL}
+	};
+	options_context = poptGetContext(NULL, argc, argv, options_table, 0);
+	gnome_init_with_popt_table("ghemical", VERSION, argc, argv, options_table, 0, &options_context);
+
 	app_cleaner.SetInstance(gnome_mdi_app::GetInstance());
+
+	if (filename != NULL && strlen(filename) != 0)
+	  {
+	    gnome_docv * docv;
+	    docv = new gnome_mm1_docv(& cout, * gnome_class_factory::GetInstance());
+	    ifstream ifile;
+	    ifile.open(filename, ios::in);
+	    if (docv->ReadStream(ifile) != false)
+	      {
+		docv->InitGnomeMDIChild();
+		docv->ParseProjectFileNameAndPath(filename);
+		docv->UpdateAllWindowTitles();
+	      }
+	    ifile.close();
+	  }
+
+	if (importname != NULL && strlen(importname) != 0)
+	  {
+	    gnome_docv * docv;
+	    docv = new gnome_mm1_docv(& cout, * gnome_class_factory::GetInstance());
+	    if (docv->ImportFile(importname, 0) != false)
+	      {
+		docv->InitGnomeMDIChild();
+		docv->ParseProjectFileNameAndPath(importname);
+		docv->UpdateAllWindowTitles();
+	      }
+	  }
+	
+	while ((importname = poptGetArg(options_context)) != NULL)
+	  {
+	    gnome_docv * docv;
+	    docv = new gnome_mm1_docv(& cout, * gnome_class_factory::GetInstance());
+	    if (docv->ImportFile(importname, 0) != false)
+	      {
+		docv->InitGnomeMDIChild();
+		docv->ParseProjectFileNameAndPath(importname);
+		docv->UpdateAllWindowTitles();
+	      }
+	  }
+	
+	poptFreeContext(options_context);
+
 	gtk_main(); gtk_exit(0);
 }
 
