@@ -58,6 +58,8 @@ bool mm2_file_open_dialog::OkEvent(const char * filename)
 	ifile.close();
 	
 	docv->ParseProjectFileNameAndPath(filename);
+	
+	gnome_mdi_child_set_name(GNOME_MDI_CHILD(docv->child), docv->GetProjectFileName(true));
 	docv->UpdateAllWindowTitles();
 	
 	docv->UpdateAllGraphicsViews();
@@ -71,6 +73,48 @@ void mm2_file_open_dialog::CancelEvent(void)
 
 /*################################################################################################*/
 
+const char mm2_file_save_dialog::title[] = "Save Project File - mm2gp";
+gnome_mm2_docv * mm2_file_save_dialog::docv = NULL;
+
+mm2_file_save_dialog::mm2_file_save_dialog(gnome_mm2_docv * p1) :
+	file_dialog(title, MM2_FILTER, p1->GetFullProjectFileName())
+{
+	docv = p1;
+}
+
+mm2_file_save_dialog::~mm2_file_save_dialog(void)
+{
+}
+
+bool mm2_file_save_dialog::OkEvent(const char * filename)
+{
+	if (docv == NULL)
+	{
+	  docv->err->ErrorMessage("mm2_file_save_dialog::OkEvent() failed");
+	  exit(EXIT_FAILURE);
+	}
+	
+  	// should check to see if file already exists...
+
+	docv->ParseProjectFileNameAndPath(filename);
+	
+	gnome_mdi_child_set_name(GNOME_MDI_CHILD(docv->child), docv->GetProjectFileName(true));
+	docv->UpdateAllWindowTitles();
+	
+	ofstream ofile;
+	ofile.open(docv->GetFullProjectFileName(), ios::out);
+	docv->WriteStream(ofile);
+	ofile.close();
+
+	return true;
+}
+
+void mm2_file_save_dialog::CancelEvent(void)
+{
+}
+
+/*################################################################################################*/
+
 const char mm2_trajfile_dialog::title[] = "Open Trajectory File - traj";
 mm2_docv * mm2_trajfile_dialog::docv = NULL;
 
@@ -104,8 +148,25 @@ void mm2_trajfile_dialog::CancelEvent(void)
 
 /*################################################################################################*/
 
+GnomeUIInfo gnome_mm2_docv::fileimport_submenu[] =
+{
+	GNOMEUIINFO_ITEM("pdb...", "pdb",
+		popup_FileImport, NULL),
+	
+	GNOMEUIINFO_END
+};
+
 GnomeUIInfo gnome_mm2_docv::file_submenu[] =
 {
+	GNOMEUIINFO_MENU_OPEN_ITEM(popup_FileOpen, NULL),
+	GNOMEUIINFO_SUBTREE("_Import", fileimport_submenu),
+	
+	GNOMEUIINFO_SEPARATOR,
+	
+	GNOMEUIINFO_MENU_SAVE_AS_ITEM(popup_FileSaveAs, NULL),
+	
+	GNOMEUIINFO_SEPARATOR,
+	
 	GNOMEUIINFO_MENU_CLOSE_ITEM(gnome_graphics_view::popup_FileClose, NULL),
 	
 	GNOMEUIINFO_END
@@ -126,20 +187,20 @@ GnomeUIInfo gnome_mm2_docv::viewscreate_submenu[] =
 {
 	GNOMEUIINFO_ITEM("New Camera", "create a new view with a new camera...",
 		gnome_graphics_view::popup_ViewsNewCam, NULL),
-		
+	
 	GNOMEUIINFO_ITEM("New View", "create a new view using the current camera...",
 		gnome_graphics_view::popup_ViewsNewView, NULL),
-		
-//	GNOMEUIINFO_SEPARATOR,
-//	
-//	GNOMEUIINFO_ITEM("TreeList", "create a tree/list view...",
-//		popup_TreeList, NULL),
-		
+	
 	GNOMEUIINFO_END
 };
 
 GnomeUIInfo gnome_mm2_docv::views_submenu[] =
 {
+	GNOMEUIINFO_ITEM("Attach/Detach This View", "attach/detach this view...",
+		gnome_graphics_view::popup_ViewsAttachDetach, NULL),
+		
+	GNOMEUIINFO_SEPARATOR,
+	
 	GNOMEUIINFO_SUBTREE("_Create", viewscreate_submenu),
 	
 	GNOMEUIINFO_ITEM("Delete This View", "remove this view...",
@@ -309,6 +370,32 @@ gnome_mm2_docv::~gnome_mm2_docv(void)
 {
 }
 
+void gnome_mm2_docv::popup_FileOpen(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
+	docv->err->Message("Sorry, this feature is not yet implemented.");
+}
+
+void gnome_mm2_docv::popup_FileImport(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
+	if (docv)
+	{
+cout << "give name of the PDB file to import ??? "; cin >> docv->buffer;
+cout << "give the chain index to be imported, or -1 to import all chains ??? "; i32s ci; cin >> ci;
+docv->ImportPDB(docv->buffer, ci); docv->UpdateAllGraphicsViews();
+	}
+}
+
+void gnome_mm2_docv::popup_FileSaveAs(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
+	gnome_mm2_docv * docv = dynamic_cast<gnome_mm2_docv *>(gv->docv);
+	if (docv) new mm2_file_save_dialog(docv);	// memory leak???
+}
+
 void gnome_mm2_docv::popup_RModeWireFrame(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
@@ -498,7 +585,9 @@ void gnome_mm2_docv::popup_CompGeomOpt(GtkWidget *, gpointer data)
 	if (docv)
 	{
 		mm2_geomopt_param param;
-		param.nsteps = 500;
+		param.treshold_nsteps = 500;
+		param.treshold_grad = 0.0;	// not yet used...
+		param.treshold_delta_e = 0.0;	// not yet used...
 		
 		docv->DoGeomOpt(param);
 	}
