@@ -50,7 +50,7 @@ bool qm1_file_open_dialog::OkEvent(const char * filename)
 		docv->InitGnomeMDIChild();
 	}
 	
-//	ifstream ifile;
+//	ifstream ifile;				THIS IS NOT YET READY...
 //	ifile.open(filename, ios::in);
 //	docv->ReadStream(ifile);
 //	ifile.close();
@@ -119,6 +119,7 @@ GnomeUIInfo gnome_qm1_docv::file_submenu[] =
 	GNOMEUIINFO_SEPARATOR,
 	
 	GNOMEUIINFO_MENU_CLOSE_ITEM(gnome_graphics_view::popup_FileClose, NULL),
+	
 	GNOMEUIINFO_END
 };
 
@@ -126,6 +127,9 @@ GnomeUIInfo gnome_qm1_docv::select_submenu[] =
 {
 	GNOMEUIINFO_ITEM("Select All", "select all atoms...",
 		gnome_graphics_view::popup_SelectAll, NULL),
+
+	GNOMEUIINFO_ITEM("Select All", "select all atoms...",
+		gnome_graphics_view::popup_SelectNone, NULL),
 		
 	GNOMEUIINFO_ITEM("Invert Selection", "invert the selection...",
 		gnome_graphics_view::popup_InvertSelection, NULL),
@@ -159,6 +163,11 @@ GnomeUIInfo gnome_qm1_docv::viewscreate_submenu[] =
 
 GnomeUIInfo gnome_qm1_docv::views_submenu[] =
 {
+	GNOMEUIINFO_ITEM("Attach/Detach This View", "attach/detach this view...",
+		gnome_graphics_view::popup_ViewsAttachDetach, NULL),
+		
+	GNOMEUIINFO_SEPARATOR,
+	
 	GNOMEUIINFO_SUBTREE("_Create", viewscreate_submenu),
 	
 	GNOMEUIINFO_ITEM("Delete This View", "remove this view...",
@@ -321,6 +330,8 @@ GnomeUIInfo gnome_qm1_docv::popupmenu_info[] =
 
 /*################################################################################################*/
 
+qm1_geomopt_param * gnome_qm1_docv::goparam = NULL;
+
 gnome_qm1_docv::gnome_qm1_docv(ostream * p1, gnome_class_factory & p2) :
 	gnome_docv(p1, p2), qm1_docv(p1, p2), docview(p1, p2), model_simple(p1, p2)
 {
@@ -351,7 +362,7 @@ void gnome_qm1_docv::popup_FileImport(GtkWidget *, gpointer data)
 	gnome_qm1_docv * docv = dynamic_cast<gnome_qm1_docv *>(gv->docv);
 	if (docv)
 	{
-		new qm1_importmm1_dialog(docv)		;
+		new qm1_importmm1_dialog(docv);
 //cout << "name of MM1 file to import ??? ";
 //cin >> docv->buffer; docv->ImportMM1(docv->buffer);
 //docv->UpdateAllGraphicsViews();
@@ -739,8 +750,10 @@ void gnome_qm1_docv::popup_CompGeomOpt(GtkWidget *, gpointer data)
 	if (docv)
 	{
 		qm1_geomopt_param param;
-		param.nsteps = 500;
-		
+		param.treshold_nsteps = 100;
+		param.treshold_grad = 1.0e-3;
+		param.treshold_delta_e = 1.0e-7;
+
 		docv->DoGeomOpt(param);
 	}
 }
@@ -760,10 +773,6 @@ void gnome_qm1_docv::popup_SetOrbital(GtkWidget *, gpointer data)
 	}
 }
 
-// temporary, for QM->MM conversion, will move into all_atoms_interface???
-// temporary, for QM->MM conversion, will move into all_atoms_interface???
-// temporary, for QM->MM conversion, will move into all_atoms_interface???
-
 void gnome_qm1_docv::popup_ConvertMM(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = gnome_graphics_view::GetGV((GtkWidget *) data);
@@ -779,46 +788,21 @@ void gnome_qm1_docv::popup_ConvertMM(GtkWidget *, gpointer data)
 	if (close_old) mm_docv->SetProjectFileName(docv->GetProjectFileName(false));
 		mm_docv->InitGnomeMDIChild();
 		
-		// start conversion qm1gp -> mm1gp.
-		// start conversion qm1gp -> mm1gp.
-		// start conversion qm1gp -> mm1gp.
+// when calling InitGnomeMDIChild(), also a graphics_view and therefore a new camera object were added.
+// is order to retain as much grahics settings as possible, copy the current view's camera settings to
+// the new project's view. WARNING!!! this is a bit ugly way to do it: in the new project, there is
+// only a single view object and a single camera object; so we just pick them! TODO : lights???
+mm_docv->camera_vector.front()->CopySettings(gv->cam);
+// an initial solution to the lights problem : remove the old light and make a new one using default
+// settings. looks good if the user has not modified the lights, but otherwise restores defaults.
+mm_docv->RemoveLight(mm_docv->light_vector.front());
+mm_docv->AddLocalLight(new directional_light(ol_static(mm_docv->camera_vector.front()->GetLocData())), mm_docv->camera_vector.front());
+
+		// convert qm1gp -> mm1gp using all_atoms_interface...
 		
-		vector<qm1_atom *> qmtab;	// store the pointers here...
-		vector<mm1_atom *> mmtab;	// ...to replicate the bonds!
+		mm_docv->aai_MakeCopy(docv);
 		
-		iter_qm1al it1 = docv->atom_list.begin();
-		while (it1 != docv->atom_list.end())
-		{
-			element tmp1 = (* it1).el;
-			fGL * tmp2 = (* it1).crd_vector[0].data;
-			
-			mm1_atom newatom(tmp1, tmp2, 1);
-			mm_docv->AddAtom(newatom);
-			
-			qmtab.push_back(& (* it1));
-			mmtab.push_back(& mm_docv->atom_list.back());
-			
-			it1++;
-		}
-		
-		iter_qm1bl it2 = docv->bond_list.begin();
-		while (it2 != docv->bond_list.end())
-		{
-			i32u index1 = 0; while (index1 < qmtab.size()) if ((* it2).atmr[0] == qmtab[index1]) break; else index1++;
-			if (index1 == qmtab.size()) { cout << "FATAL ERROR : could not find atom #1." << endl; exit(EXIT_FAILURE); }
-			
-			i32u index2 = 0; while (index2 < qmtab.size()) if ((* it2).atmr[1] == qmtab[index2]) break; else index2++;
-			if (index2 == qmtab.size()) { cout << "FATAL ERROR : could not find atom #2." << endl; exit(EXIT_FAILURE); }
-			
-			bondtype tmp1 = (* it2).bt;
-			
-			mm1_bond newbond(mmtab[index1], mmtab[index2], tmp1);
-			mm_docv->AddBond(newbond);
-			
-			it2++;
-		}
-		
-		// conversion is now ready.
+		// ...the conversion is now ready.
 		
 		if (close_old) gnome_mdi_remove_child(gnome_mdi_app::mdi, GNOME_MDI_CHILD(gv->docv->child), TRUE);
 	}
@@ -829,10 +813,17 @@ void gnome_qm1_docv::popup_ConvertMM(GtkWidget *, gpointer data)
 
 void gnome_qm1_docv::GeomOptGetParam(qm1_geomopt_param & param)
 {
-	cout << "how many cg-steps (" << param.nsteps << " might be Ok, -1 = cancel) ??? ";
-	cin >> param.nsteps;
+	glade_modal_dialog_owner = this;	// requires modal dialogs!!!
 	
-	param.confirm = !(param.nsteps < 0);
+	// here we just set "param" to a static variable,
+	// where it's then available in glade code...
+	
+	goparam = & param;
+	
+	GtkWidget * dialog = create_a1_dialog_geomopt();
+	gnome_dialog_run_and_close(GNOME_DIALOG(dialog));
+	
+	goparam = NULL;
 }
 
 /*################################################################################################*/
