@@ -43,6 +43,21 @@ docview * gnome_view::GetDV(void)
 
 void gnome_view::SetTitle(const char * p1)
 {
+	// if this a "detached" view, we should do the title setting differently than usual "attached" cases.
+	// only gnome_graphics_view objects can be "detached". ok, this is a bit ugly solution, but see
+	// gnome_graphics_view::popup_ViewsAttachDetach() for more comments...
+	
+	gnome_graphics_view * gv = dynamic_cast<gnome_graphics_view *>(this);
+	if (gv != NULL && gv->detached != NULL)
+	{
+		char extended_title[256];
+		ostrstream str(extended_title, sizeof(extended_title));
+		str << docv->GetProjectFileName(true) << " : " << p1 << ends;
+		
+		gtk_window_set_title(GTK_WINDOW(gv->detached), extended_title);
+		return;
+	}
+	
 	gtk_label_set_text(GTK_LABEL(label_widget), p1);
 }
 
@@ -66,13 +81,11 @@ gnome_ogl_view::gnome_ogl_view(gnome_docv * p1) : gnome_view(p1), ogl_view()
 	};
 	
 	view_widget = gtk_gl_area_new(attrlst);
-//	gtk_widget_set_usize(GTK_WIDGET(view_widget), 100, 100);
 	gtk_widget_set_usize(GTK_WIDGET(view_widget), 400, 400);	// how to set DEFAULT sizes !?!?!?
 	
 	int events = GDK_EXPOSURE_MASK;
 	events |= GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK;
 	events |= GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK;
-	
 	gtk_widget_set_events(GTK_WIDGET(view_widget), events);
 
 	gtk_signal_connect(GTK_OBJECT(view_widget),
@@ -92,7 +105,7 @@ gnome_ogl_view::gnome_ogl_view(gnome_docv * p1) : gnome_view(p1), ogl_view()
 		
 	gtk_signal_connect(GTK_OBJECT(view_widget),
 		"configure_event", GTK_SIGNAL_FUNC(ConfigureHandler), NULL);
-		
+	
 	gtk_widget_show(GTK_WIDGET(view_widget));
 }
 
@@ -268,12 +281,23 @@ gint gnome_ogl_view::MotionNotifyHandler(GtkWidget * widget, GdkEventMotion * ev
 gnome_graphics_view::gnome_graphics_view(gnome_docv * p1, camera * p2) :
 	gnome_ogl_view(p1), graphics_view(p2), ogl_view()
 {
+	detached = NULL;
 }
 
 gnome_graphics_view::~gnome_graphics_view(void)
 {
 }
 
+gint gnome_graphics_view::DetachedDeleteHandler(GtkWidget *, GdkEvent *)
+{
+	// when we create detached view windows as GTK_WINDOW_TOPLEVEL, the window will have the "close" button
+	// at titlebar. now if the user presses the "close" button, the window-closing sequence will start.
+	// we will grab the resulting delete_event here and return TRUE, that will deny the user's request to
+	// close the window. the user should use the stardard popup-way of closing the window...
+	
+	return TRUE;
+}
+
 /*##############################################*/
 /*##############################################*/
 
@@ -297,6 +321,13 @@ void gnome_graphics_view::popup_SelectAll(GtkWidget *, gpointer data)
 	GetGV((GtkWidget *) data)->docv->SelectAll();
 }
 
+void gnome_graphics_view::popup_SelectNone(GtkWidget *, gpointer data)
+{
+	GetGV((GtkWidget *) data)->docv->SelectAll();
+	GetGV((GtkWidget *) data)->docv->InvertSelection();
+	//GetGV((GtkWidget *) data)->docv->SelectNone();	// the original...
+}
+
 void gnome_graphics_view::popup_InvertSelection(GtkWidget *, gpointer data)
 {
 	GetGV((GtkWidget *) data)->docv->InvertSelection();
@@ -406,6 +437,31 @@ void gnome_graphics_view::popup_RenderQuickUpdate(GtkWidget *, gpointer data)
 	gv->quick_update = !gv->quick_update;
 }
 
+void gnome_graphics_view::popup_ViewsAttachDetach(GtkWidget *, gpointer data)
+{
+	gnome_graphics_view * gv = GetGV((GtkWidget *) data);
+	
+// this function offers attaching/detaching to/from the docv notebook. it perhaps would be nice
+// if ALL views had this possibility, but that turned out to be complicated. the graphics views benefit
+// most from this; also the treelist view should remain in notebook (or the notebook might become empty).
+// anyways, more elegant way would be to just take the widget out of the notebook page, and then connect
+// it to a new GtkWindow. however, it seems not be that easy (at least in GTK v1.2) because a call to
+// the gtk_notebook_remove_page() seems to destroy the page widget.
+
+	// first we create a new view using the requested mode...
+	
+	bool flag = (gv->detached == NULL);
+	graphics_view * new_gv = gv->docv->AddGraphicsView(gv->cam, flag);
+	
+	new_gv->render = gv->render;	// try to retain as much graphics settings as possible.
+	new_gv->label = gv->label;
+	
+	// ...and then simply destroy the old one; calling popup_ViewsDeleteView()
+	// below this is the same as selecting "Delete This View" from the popup-menu.
+	
+	popup_ViewsDeleteView(NULL, data);
+}
+
 void gnome_graphics_view::popup_ViewsNewCam(GtkWidget *, gpointer data)
 {
 	gnome_graphics_view * gv = GetGV((GtkWidget *) data);
@@ -428,15 +484,24 @@ void gnome_graphics_view::popup_ViewsDeleteView(GtkWidget *, gpointer data)
 	// the project, RemoveGraphicsView() will just pop up an error message and skip
 	// the removal. so, we use IsThisLastGraphicsView() to detect this case...
 	
+	// now we also have those "detached" windows, and we have to handle them separately.
+	
 	if (!gv->docv->IsThisLastGraphicsView(gv))
 	{
-		gint page = gtk_notebook_page_num(GTK_NOTEBOOK(gv->docv->notebook), gv->view_widget);
-		gtk_notebook_remove_page(GTK_NOTEBOOK(gv->docv->notebook), page);
+		if (gv->detached != NULL)
+		{
+			gtk_widget_destroy(GTK_WIDGET(gv->detached));
+		}
+		else
+		{
+			gint page = gtk_notebook_page_num(GTK_NOTEBOOK(gv->docv->notebook), gv->view_widget);
+			gtk_notebook_remove_page(GTK_NOTEBOOK(gv->docv->notebook), page);
+		}
 	}
 	
 // perhaps a more elegant way would be to change "add/remove view"-funtions of docview into virtual functions,
 // and then override the default behaviour at, say, here in gnome classes (where we would do add/remove operations
-// for out GtkNotebook). HOWEVER, it seems to have some side-effects; just lost 4h trying to figure it out...  :(
+// for out GtkNotebook). HOWEVER, it seems to have some side-effects; just lost 4 h trying to figure it out...  :(
 
 	gv->docv->RemoveGraphicsView(gv);
 }
@@ -514,6 +579,40 @@ void gnome_graphics_view::ExposeEvent(void)
 
 /*################################################################################################*/
 
+gnome_plot1d_view::gnome_plot1d_view(model_simple * p1, i32s p2, i32s p3) :
+	gnome_ogl_view(dynamic_cast<gnome_docv *>(p1)), plot1d_view(p1, p2, p3), ogl_view()
+{
+	gtk_label_set_text(GTK_LABEL(label_widget), "1D plot view");
+}
+
+gnome_plot1d_view::~gnome_plot1d_view(void)
+{
+}
+
+void gnome_plot1d_view::ExposeEvent(void)
+{
+	SetCurrent(); Render(); glFlush();
+	gtk_gl_area_swapbuffers(GTK_GL_AREA(view_widget));	// older way??? see gnome_graphics_view::ExposeEvent()
+}
+
+gnome_plot2d_view::gnome_plot2d_view(model_simple * p1, i32s p2, i32s p3) :
+	gnome_ogl_view(dynamic_cast<gnome_docv *>(p1)), plot2d_view(p1, p2, p3), ogl_view()
+{
+	gtk_label_set_text(GTK_LABEL(label_widget), "2D plot view");
+}
+
+gnome_plot2d_view::~gnome_plot2d_view(void)
+{
+}
+
+void gnome_plot2d_view::ExposeEvent(void)
+{
+	SetCurrent(); Render(); glFlush();
+	gtk_gl_area_swapbuffers(GTK_GL_AREA(view_widget));	// older way??? see gnome_graphics_view::ExposeEvent()
+}
+
+/*################################################################################################*/
+
 gnome_enlevdiag_view::gnome_enlevdiag_view(qm1_mdl * p1) :
 	gnome_ogl_view(dynamic_cast<gnome_docv *>(p1)), enlevdiag_view(p1), ogl_view()
 {
@@ -527,8 +626,7 @@ gnome_enlevdiag_view::~gnome_enlevdiag_view(void)
 void gnome_enlevdiag_view::ExposeEvent(void)
 {
 	SetCurrent(); Render(); glFlush();
-//	gtk_gl_area_swap_buffers(GTK_GL_AREA(view_widget));		// newer way???
-	gtk_gl_area_swapbuffers(GTK_GL_AREA(view_widget));		// older way???
+	gtk_gl_area_swapbuffers(GTK_GL_AREA(view_widget));	// older way??? see gnome_graphics_view::ExposeEvent()
 }
 
 /*################################################################################################*/
@@ -780,17 +878,54 @@ prefs * gnome_class_factory::ProducePrefs(void)
         return new gnome_prefs();
 }
 
-graphics_view * gnome_class_factory::ProduceGraphicsView(docview * docv1, camera * cam)
+graphics_view * gnome_class_factory::ProduceGraphicsView(docview * docv1, camera * cam, bool detach)
 {
 	gnome_docv * docv2 = dynamic_cast<gnome_docv *>(docv1);
 	
 	gnome_graphics_view * gv = new gnome_graphics_view(docv2, cam);
-	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), gv->view_widget, gv->label_widget);
-	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
+	
+	if (detach)
+	{
+		gv->detached = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		gtk_window_set_default_size(GTK_WINDOW(gv->detached), 400, 400);
+		gtk_container_add(GTK_CONTAINER(gv->detached), gv->view_widget);
+		
+		gtk_signal_connect(GTK_OBJECT(gv->detached),
+			"delete_event", GTK_SIGNAL_FUNC(gnome_graphics_view::DetachedDeleteHandler), NULL);
+		
+		gtk_widget_show(gv->detached);
+	}
+	else
+	{
+		gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), gv->view_widget, gv->label_widget);
+		gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
+	}
 	
 	return gv;
 }
 
+plot1d_view * gnome_class_factory::ProducePlot1DView(docview * docv1, i32s ud1, i32s ud2)
+{
+	gnome_docv * docv2 = dynamic_cast<gnome_docv *>(docv1);
+	
+	gnome_plot1d_view * plotv = new gnome_plot1d_view(docv1, ud1, ud2);
+	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), plotv->view_widget, plotv->label_widget);
+	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
+	
+	return plotv;
+}
+
+plot2d_view * gnome_class_factory::ProducePlot2DView(docview * docv1, i32s ud1, i32s ud2)
+{
+	gnome_docv * docv2 = dynamic_cast<gnome_docv *>(docv1);
+	
+	gnome_plot2d_view * plotv = new gnome_plot2d_view(docv1, ud1, ud2);
+	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), plotv->view_widget, plotv->label_widget);
+	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page.
+	
+	return plotv;
+}
+
 enlevdiag_view * gnome_class_factory::ProduceEnLevDiagView(qm1_mdl * mdl)
 {
 	gnome_docv * docv = dynamic_cast<gnome_docv *>(mdl);
@@ -808,6 +943,7 @@ treelist_view * gnome_class_factory::ProduceTreeListView(docview * docv1)
 	
 	gnome_treelist_view * tlv = new gnome_treelist_view(docv2);
 	gtk_notebook_append_page(GTK_NOTEBOOK(docv2->notebook), tlv->view_widget, tlv->label_widget);
+//	gtk_notebook_set_page(GTK_NOTEBOOK(docv2->notebook), -1);	// activate the last page. // currently has no effect...
 	
 	return tlv;
 }
